import React, { useEffect, useState } from "react";
import Board from "@lourenci/react-kanban";
import "@lourenci/react-kanban/dist/styles.css";
import {
  IKanbanProps,
  IKanbanWebListObject,
  IKanbanWebListObjectDS,
  IKanbanWebCardObject,
} from "./types";
import { v4 as uuidv4 } from "uuid";
import {
  parseDataToLibraryDataStructure,
  parseLibraryToListDataStructure,
} from "./webUtils";

export default (props: IKanbanProps) => {
  const [data, setData] = useState(parseDataToLibraryDataStructure(props.data));

  useEffect(() => {
    setData(parseDataToLibraryDataStructure(props.data));
  }, [props.data]);

  useEffect(() => {
    if (props.onChange) {
      props.onChange(
        data.columns.map((column) => parseLibraryToListDataStructure(column))
      );
    }
  }, [data]);

  let defaultProps = {};

  if (props.titleComponent) {
    defaultProps = {
      ...defaultProps,
      renderColumnHeader: (list: IKanbanWebListObject, listBag: any) => {
        const TitleComponent = props.titleComponent;
        if (TitleComponent) {
          return (
            <TitleComponent
              title={list.title}
              onTitleUpdate={(updatedTitle) => {
                if (!props.readOnly) {
                  listBag.renameColumn(updatedTitle);
                  setData({
                    ...data,
                    columns: data.columns.map((list) => {
                      if (list.id === list.id) {
                        return { ...list, title: updatedTitle };
                      }
                      return list;
                    }),
                  });
                }
              }}
            />
          );
        } else {
          <></>;
        }
      },
    };
  }

  if (props.cardComponent) {
    defaultProps = {
      ...defaultProps,
      renderCard: (card: IKanbanWebCardObject, cardBag: any) => {
        const CardComponent = props.cardComponent;
        if (CardComponent) {
          return (
            <CardComponent
              id={card.id}
              listId={cardBag.column.id}
              title={card.title}
              data={{ description: card.description }}
              onTitleUpdate={(updatedTitle) => {
                if (!props.readOnly) {
                  cardBag.renameColumn(updatedTitle);
                  setData({
                    ...data,
                    columns: data.columns.map((list) => {
                      if (list.id === list.id) {
                        return { ...list, title: updatedTitle };
                      }
                      return list;
                    }),
                  });
                }
              }}
            />
          );
        } else {
          return <></>;
        }
      },
    };
  }

  if (!props.readOnly) {
    if (props.canAddList) {
      defaultProps = {
        ...defaultProps,
        onNewColumnConfirm: (newColumn: IKanbanWebListObject) => {
          return { id: uuidv4(), ...newColumn };
        },
        onColumnNew: (updatedData: IKanbanWebListObjectDS) => {
          setData(updatedData);
        },
        allowAddColumn: true,
      };
    }

    if (props.canAddCard) {
      defaultProps = {
        ...defaultProps,
        allowAddCard: true,
        onNewCardConfirm: (newCard: IKanbanWebCardObject) => {
          return { id: uuidv4(), ...newCard };
        },
        onCardNew: (updatedData: IKanbanWebListObjectDS) => {
          setData(updatedData);
        },
      };
    }

    defaultProps = {
      ...defaultProps,
      allowRenameColumn: true,
      onColumnRename: (updatedData: IKanbanWebListObjectDS) => {
        setData(updatedData);
      },
    };

    defaultProps = {
      ...defaultProps,
      onColumnDragStart: (subject: IKanbanWebListObject, source: any) => {
        if (props.onDrag) {
          props.onDrag({
            type: "list",
            sourceId: subject.id,
            from: source.fromPosition,
          });
        }
      },
      onColumnDragEnd: (
        updatedData: IKanbanWebListObjectDS,
        subject: IKanbanWebListObject,
        source: any,
        destination: any
      ) => {
        if (props.onDrop) {
          props.onDrop({
            type: "list",
            sourceId: subject.id,
            from: source.fromPosition,
            to: destination.toPosition,
          });
        }
        setData(updatedData);
      },
      onCardDragStart: (subject: IKanbanWebListObject, source: any) => {
        if (props.onDrag) {
          props.onDrag({
            type: "card",
            sourceId: subject.id,
            from: source.fromPosition,
            parentFrom: source.fromColumnId,
          });
        }
      },
      onCardDragEnd: (
        updatedData: IKanbanWebListObjectDS,
        subject: IKanbanWebCardObject,
        source: any,
        destination: any
      ) => {
        if (props.onDrop) {
          props.onDrop({
            type: "card",
            sourceId: subject.id,
            from: source.fromPosition,
            to: destination.toPosition,
            parentFrom: source.fromColumnId,
            parentTo: destination.toColumnId,
          });
        }
        setData(updatedData);
      },
    };

    defaultProps = {
      ...defaultProps,
      onColumnRemove: (updatedData: IKanbanWebListObjectDS) => {
        setData(updatedData);
      },
      allowRemoveColumn: true,
    };
    defaultProps = {
      ...defaultProps,
      onCardRemove: (updatedData: IKanbanWebListObjectDS) => {
        setData(updatedData);
      },
      allowRemoveCard: true,
    };
  } else {
    defaultProps = {
      ...defaultProps,
      disableColumnDrag: true,
      disableCardDrag: true,
    };
  }
  return (
    <>
      <h3>{props.title}</h3>
      <Board {...defaultProps} initialBoard={data} />
    </>
  );
};
