import React from "react";
import Board, {
  changeColumn,
  moveColumn,
  moveCard,
  removeColumn,
  removeCard,
  addCard,
} from "react-kanban";

import {
  IKanbanCardObject,
  IKanbanColumnObject,
  IKanbanBoardProps,
} from "./types";
import { v4 as uuidv4 } from "uuid";
import KanbanColumnHeader from "./KanbanColumnHeader";
import KanbanCard from "./KanbanCard";

export default (props: IKanbanBoardProps) => {
  let defaultProps: any = {
    renderColumnHeader: (columnProps: IKanbanColumnObject) => {
      return (
        <KanbanColumnHeader
          {...columnProps}
          canAddCard={props.canAddCard}
          theme={props.theme}
          readOnly={props.readOnly}
          onRemoveColumn={() => {
            if (props.onColumnDelete) {
              props.onColumnDelete(columnProps.id);
            }
            props.onChange(removeColumn(props.data, columnProps));
          }}
          onAddCard={() => {
            const newCard = {
              id: uuidv4(),
              title: "New Card",
              description: undefined,
              image: "https://picsum.photos/200/300",
            };
            if (props.onCardCreate) {
              props.onCardCreate(columnProps.id, newCard);
            }
            props.onChange(
              addCard(props.data, columnProps, newCard, { on: "top" })
            );
          }}
          onTitleUpdate={(updatedTitle) => {
            const updatedColumn = changeColumn(props.data, columnProps, {
              title: updatedTitle,
            });
            if (props.onColumnChange) {
              props.onColumnChange(updatedColumn);
            }
            props.onChange(updatedColumn);
          }}
        />
      );
    },
    renderCard: (
      cardProps: IKanbanCardObject,
      extraOptions: { column: IKanbanColumnObject }
    ) => {
      return (
        <KanbanCard
          {...cardProps}
          theme={props.theme}
          readOnly={props.readOnly}
          onTitleUpdate={(updatedTitle) => {
            const updatedCard = { ...cardProps, title: updatedTitle };
            if (props.onCardChange) {
              props.onCardChange(extraOptions.column.id, updatedCard);
            }
            props.onChange(
              changeColumn(props.data, extraOptions.column, {
                cards: extraOptions.column.cards.map((card) => {
                  if (card.id === cardProps.id) {
                    return updatedCard;
                  }
                  return card;
                }),
              })
            );
          }}
          onCardRemove={() => {
            if (props.onCardDelete) {
              props.onCardDelete(extraOptions.column.id, cardProps.id);
            }
            props.onChange(
              removeCard(props.data, extraOptions.column, cardProps)
            );
          }}
        />
      );
    },
  };

  if (!props.readOnly) {
    defaultProps = {
      ...defaultProps,
      onColumnDragStart: (subject: IKanbanColumnObject, source: any) => {
        if (props.onColumnIsDragging) {
          props.onColumnIsDragging({
            type: "column",
            sourceId: subject.id,
            from: source.fromPosition,
          });
        }
      },
      onColumnDragEnd: (
        subject: IKanbanColumnObject,
        source: any,
        destination: any
      ) => {
        if (props.onDrop) {
          props.onDrop({
            type: "column",
            sourceId: subject.id,
            from: source.fromPosition,
            to: destination.toPosition,
          });
        }

        props.onChange(
          moveColumn(
            props.data,
            { fromPosition: source.fromPosition },
            { toPosition: destination.toPosition }
          )
        );
      },
      onCardDragStart: (subject: IKanbanCardObject, source: any) => {
        if (props.onCardIsDragging) {
          props.onCardIsDragging({
            type: "card",
            sourceId: subject.id,
            from: source.fromPosition,
            parentFrom: source.fromColumnId,
          });
        }
      },
      onCardDragEnd: (
        subject: IKanbanCardObject,
        source: any,
        destination: any
      ) => {
        if (props.onDrop) {
          props.onDrop({
            type: "card",
            sourceId: subject.id,
            from: source.fromPosition,
            to: destination.toPosition,
            parentFrom: source.fromColumnId,
            parentTo: destination.toColumnId,
          });
        }
        props.onChange(
          moveCard(
            props.data,
            {
              fromPosition: source.fromPosition,
              fromColumnId: source.fromColumnId,
            },
            {
              toPosition: destination.toPosition,
              toColumnId: destination.toColumnId,
            }
          )
        );
      },
    };

    defaultProps = {
      ...defaultProps,
      onCardRemove: (column: IKanbanColumnObject, card: IKanbanCardObject) => {
        if (props.onCardDelete) {
          props.onCardDelete(column.id, card.id);
        }
        props.onChange(removeCard(props.data, column, card));
      },
      allowRemoveCard: true,
    };
  } else {
    defaultProps = {
      ...defaultProps,
      disableColumnDrag: true,
      disableCardDrag: true,
    };
  }
  return <Board {...defaultProps}>{props.data}</Board>;
};
