import _ from 'lodash'
import express from 'express'

import { prisma } from "@sc/api/db/prisma/generated/prisma-client";
import { cloneCampaign, publishCampaign, getCampaign } from './gql'// '@sc/api/gql/campaigns';
import { getPage } from '../pages/gql'

import { client, prismaClient } from '../client'
import { getUserIdFromToken, getUserInfoFromToken } from '../sessions'


const campaigns = express.Router()

// get campaigns
/**
 * @swagger
 * /campaigns:
 *  get:
 *    tags:
 *      - "Campaigns"
 *    summary: "Get a list of all the campaigns created by the logged in user"
 *    description: ""
 *    consumes:
 *    - "application/json"
 *    produces:
 *    - "application/json"
 *    responses:
 *      "200":
 *        description: ""
 *        schema:
 *          type: "object"
 *          properties:
 *              data:
 *                  type: "array"
 *                  items: 
 *                    $ref: "#definitions/Campaign"
 */
campaigns.get('/', async (req, res) => {
    try {
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token)
        console.log({ userId })

        const response = await prisma.campaigns({ where: { user: { id: userId } } });
        res.status(200).send({ data: response })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

// create new campaign
/**
 * @swagger
 * /campaigns:
 *   post:
 *     tags:
 *       - "Campaigns"
 *     summary: "Create a new campaign for the active user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/Campaign"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          $ref: "#definitions/Campaign"
 */
campaigns.post('/', async (req, res) => {
    try {
        const { authorization } = req.headers;
        const { body } = req;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token)
        const { appId } = await getUserInfoFromToken(token);

        const { name, screenshot, scriptTags, type, domainId } = body;

        let data = { deleted: false };
        if (name) data.name = name;
        if (screenshot) data.screenshot = screenshot;
        if (scriptTags) data.scriptTags = scriptTags;
        if (type) data.type = type;

        if (userId) data.user = { connect: { id: userId } }
        if (domainId) {
            data.domainId = { connect: { id: domainId } }
        } else {
            // get default domain id
            const domains = await prisma.domains({ where: { user: { id: userId } } });
            const { defaultDomain } = await prisma.app({ id: appId });
            const { id } = domains.filter(domain => domain.name === `${subdomain}.${defaultDomain}`)
            data.domainId = { connect: { id } };
        }

        const response = await prisma.createCampaign(data);
        res.status(200).send(response)
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

/**
 * @swagger
 * /campaigns/{id}:
 *   get:
 *     tags:
 *       - "Campaigns"
 *     summary: "Get details about a specific campaign"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *              data:
 *                  $ref: "#definitions/Campaign"
 */
campaigns.get('/:id', async (req, res) => {
    try {
        const { body, params } = req;
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token);

        const { data: { campaigns } } = await prismaClient.query({
            query: getCampaign,
            variables: { id: params.id, userId },
            fetchPolicy: "network-only"
        })

        if (campaigns.length) res.status(200).send({ data: _.head(campaigns) })
        else res.status(500).send({ error: "No campaign was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

/**
 * @swagger
 * /campaigns/{id}:
 *  delete:
 *    tags:
 *      - "Campaigns"
 *    summary: "Remove a campaign"
 *    description: ""
 *    consumes:
 *    - "application/json"
 *    produces:
 *    - "application/json"
 *    responses:
 *      "200":
 *        description: ""
 *        schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "The campaign was deleted"
 */
campaigns.delete('/:id', async (req, res) => {
    try {
        const { params } = req;
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token);

        const campaigns = await prisma.updateManyCampaigns({ where: { id: params.id, user: { id: userId } }, data: { deleted: true } })

        if (campaigns.count) res.status(200).send({ message: "The campaign was deleted" })
        else res.status(500).send({ error: "No campaign was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

/**
 * @swagger
 * /campaigns/{id}:
 *   patch:
 *     tags:
 *       - "Campaigns"
 *     summary: "Update a campaign"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/Campaign"
 *     responses:
 *      "200":
 *        description: ""
 *        schema:
 *          $ref: "#definitions/Campaign"
 */
campaigns.patch('/:id', async (req, res) => {
    try {
        const { body, params } = req;
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token);

        const campaigns = await prisma.updateManyCampaigns({ where: { id: params.id, user: { id: userId } }, data: body })

        if (campaigns.count) {
            res.status(200).send({ message: "The campaign was updated" })
        } else res.status(500).send({ error: "No campaign was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

/**
 * @swagger
 * /campaigns/{id}/publish:
 *   post:
 *     tags:
 *       - "Campaigns"
 *     summary: "Publish an entire campaign"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            webSiteUrl: 
 *              type: "string"
 *              example: "http://subdomain.tenminutefunnels.com"
 *            statusCode: 
 *              type: "integer"
 *              example: "200"
 */
campaigns.post('/:id/publish', async (req, res) => {
    try {
        const { params, body } = req;
        const campaignId = params.id;
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false

        const userId = await getUserIdFromToken(token);
        const user = await getUserInfoFromToken(token);
        const { appId } = user;
        // campaign exists for that uesr?
        // const campaigns = await prisma.campaigns({ where: { id: campaignId, user: { id: userId } } })
        const { data: { campaigns } } = await prismaClient.query({
            query: getCampaign,
            variables: { id: campaignId, userId },
            fetchPolicy: "network-only"
        })

        if (campaigns.length) {

            const { objects, id, type, scriptTags, domain } = _.head(campaigns);
            const { defaultDomain, cloudfrontDistribution } = await prisma.app({ id: appId });
            const { subdomain } = user;

            const parentDomain = `${subdomain}.${defaultDomain}`;

            const pageObjects = objects.filter((obj) => !obj.deleted && _.has(obj, "page.slug"));

            const getNextPage = (pageId) => {

                // 1. get list of page objects
                // const objects = this.props.getCampaignQuery.campaign.objects;
                const pages = objects.filter(
                    (obj) => !obj.deleted && _.has(obj, "page.slug")
                );

                // 2. find the object associated with the provided pageId
                if (pages) {
                    const theObject = pages.filter((obj) => obj.page.id === pageId);

                    if (theObject.length) {
                        const object = theObject[0]; // this is the object associated with the page I'm using to find the nextPage's slug

                        // 3. in that object, find the first matching connectTo objId that happens to be
                        // a page (find out if any of the connectTo ids match any of the ids in the
                        // pages object array)
                        if (object.connectTo) {
                            const nextPageObj = object.connectTo.filter((itm) => {
                                const objId = itm.id; // this is the object id in the connectTo array
                                return pages.filter((pg) => pg.id === objId) && !itm.deleted;
                            });

                            // 4. return the slug of that matching object
                            // console.log(pageId, { pages }, { nextPageObj });
                            if (nextPageObj.length) {
                                const thePages = _.head(
                                    _.filter(pages, (pg) => pg.id === nextPageObj[0].id)
                                );
                                return _.get(thePages, "page.slug", false);
                            }
                            return false;
                        }
                        return false;
                    }
                }

                return false;
            }

            let returnThisObjectArray = [];

            for (const obj of pageObjects) {
                //  2. fetch page for each object
                const { page } = obj;

                //   await triggerAsyncHook(
                //     "onNodePublish",
                //     { type: obj.type, mode: "campaign" },
                //     { item: obj },
                //     {
                //       updateObjectSettings: async (variables) => await updateObjectSettings({ variables }),
                //       updatePageSettings: async (variables) => await updatePageSettings({ variables }),
                //     }
                //   );


                const { data: { pages } } = await prismaClient.query({ query: getPage, variables: { pageId: page.id, userId } });
                const myPage = _.head(pages);

                const {
                    content,
                    homePageForDomain,
                    errorPageForDomain,
                    id,
                    pageSettings,
                    slug,
                    type,
                } = myPage;

                //  3. convert to json using json.stringify
                const seoPayload = _.get(pageSettings, "seo", {});
                const emailPayload = _.get(pageSettings, "email", {});
                const merchantPayload = _.get(pageSettings, "merchant", {});
                const trackingPayload = _.get(pageSettings, "tracking", {});
                // CSS
                // Exit Popup


                const isHomePage = homePageForDomain !== null;
                const is404 = errorPageForDomain !== null;

                // console.log({ id, slug, type, seoPayload, emailPayload, merchantPayload, trackingPayload, isHomePage, is404 })

                const pageObject = {
                    content: JSON.stringify(content),
                    id,
                    slug,
                    type,
                    seoPayload: JSON.stringify(seoPayload),
                    emailPayload: JSON.stringify(emailPayload),
                    merchantPayload: JSON.stringify(merchantPayload),
                    trackingPayload: JSON.stringify(trackingPayload),
                    isHomePage,
                    is404,
                    launchDate: new Date().toISOString(),
                    nextPage: getNextPage(id) || "#",
                };

                returnThisObjectArray.push(pageObject);
            }

            const pageObjectJson = JSON.stringify(returnThisObjectArray);

            //  4. finish building campaign publish graphQL payload
            const variables = {
                campaignId: id,
                objects: pageObjectJson,
                trackingPayload: scriptTags || "{}",
                launchDate: new Date().toISOString(),
                type,
                parentDomain,
                cloudfrontDistribution
            };

            const { data } = await prismaClient.mutate({
                mutation: publishCampaign,
                variables
            })

            const firstBuildableObject = _.head(objects.filter((itm) => itm.type === "PageComponent" && !itm.deleted));
            const showSlug = _.head(_.filter(objects, (obj) => _.get(obj, "page.homePageForDomain", false) !== null && !obj.deleted));

            const slug = _.has(showSlug, "page.homePageForDomain.id") ? showSlug.page.slug : firstBuildableObject.page.slug;

            //  6. return success/fail response
            const publishedDomain = _.get(domain, "name", parentDomain);
            const url = `http://${publishedDomain}/${slug}`;
            const response = { ..._.get(data, 'publishCampaign', { error: "An error occurred while publishing" }), webSiteUrl: url }

            res.status(200).send(response);
        } else res.status(500).send({ error: "No campaign was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
})

/**
 * @swagger
 * /campaigns/{id}/clone:
 *   post:
 *     tags:
 *       - "Campaigns"
 *     summary: "Create a clone of an existing campaign"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *         type: "object"
 *         properties:
 *          userId:
 *            type: "string"
 *            description: "(optional) The id of the user you want to send the campaign to"
 *          name:
 *            type: "string"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *           type: "object"
 *           properties:
 *            statusCode:
 *              type: "integer"
 *              example: 200
 *            message:
 *              type: "string"
 *              example: "New Campaign Created"
 *            response:
 *              type: "object"
 *              $ref: "#definitions/Campaign"
 */
campaigns.post('/:id/clone', async (req, res) => {
    try {
        const { params, body } = req;
        const campaignId = params.id;

        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false

        const userId = await getUserIdFromToken(token);

        // campaign exists for that uesr?
        const campaigns = await prisma.campaigns({ where: { id: campaignId, user: { id: userId } } })

        if (campaigns.length) {
            const variables = { ...body, userId: _.get(body, 'userId', userId), id: params.id };

            const data = await prismaClient.mutate({
                mutation: cloneCampaign,
                variables
            })

            res.status(200).send(data)
        } else res.status(500).send({ error: "No campaign was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
})

export default campaigns;