import _ from 'lodash'
import express from 'express'
import { prisma } from "@sc/api/db/prisma/generated/prisma-client";

import { createSSLOnServer, createDomainOnServer, deleteDomainOnServer } from './gql';
import { getUserIdFromToken, getUserInfoFromToken } from '../sessions';
import { prismaClient, client } from '../client'

const domains = express.Router()

// get domains
/**
 * @swagger
 * /domains:
 *   get:
 *     tags:
 *       - "Domains"
 *     summary: "Get a list of all the domain names added by the logged in user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *              data:
 *                  type: "array"
 *                  items: 
 *                    $ref: "#definitions/Domain"
 */
domains.get('/', async (req, res) => {
  try {
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token)

    const response = await prisma.domains({ where: { user: { id: userId } } });
    res.status(200).send({ data: response })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// create new domain
/**
 * @swagger
 * /domains:
 *   post:
 *     tags:
 *       - "Domains"
 *     summary: "Create a new domain name for the active user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/Domain"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          $ref: "#definitions/Domain"
 */
domains.post('/', async (req, res) => {
  try {
    const { authorization } = req.headers;
    const { body } = req;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token)
    const user = await getUserInfoFromToken(token);
    const { email } = user;

    const { campaignId, name } = body;
    let data = {};
    if (name) data.name = name;
    if (userId) data.user = { connect: { id: userId } }
    if (campaignId) data.user = { connect: { id: campaignId } }
    const result = await prisma.createDomain(data);

    // trigger create domain on server graphql script
    await prismaClient.mutate({
      mutation: createDomainOnServer,
      variables: { domain: name, ssl: false, email, proxyPass: "false" }
    })

    res.status(200).send(result)
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// get domain
/**
 * @swagger
 * /domains/{id}:
 *   get:
 *     tags:
 *       - "Domains"
 *     summary: "Get details about a specific domain name"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *              data:
 *                $ref: "#definitions/Domain"
 */
domains.get('/:id', async (req, res) => {
  try {
    const { params } = req;
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token);

    const response = await prisma.domains({ where: { id: params.id, user: { id: userId } } });

    if (response.length) res.status(200).send({ data: _.head(response) })
    else res.status(500).send({ error: "No domain was found" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// delete a domain
/**
 * @swagger
 * /domains/{id}:
 *   delete:
 *     tags:
 *       - "Domains"
 *     summary: "Remove a domain name from a user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: "" 
 *         schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "The domwain was deleted"
 */
domains.delete('/:id', async (req, res) => {
  try {
    const { params } = req;
    // const { domainId } = params;
    const domainId = params.id;
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token);

    // const domains = await prisma.updateManyDomains({ where: { id: params.id, user: { id: userId } }, data: { deleted: true } })
    const response = await prisma.domains({ where: { id: params.id, user: { id: userId } } });
    if (response.length) {

      await prisma.deleteDomain({ id: domainId })
      await prismaClient.mutate({
        mutation: deleteDomainOnServer,
        variables: { domain: _.get(_.head(response), 'name') }
      })

      res.status(200).send({ message: "The domain was deleted" })
    }
    else res.status(500).send({ error: "No domain was found" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// update a domain
/**
 * /domains/{id}:
 *   patch:
 *     tags:
 *       - "Domains"
 *     summary: "Update a domain"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "The domain was updated"
 */
// domains.patch('/:id', async (req, res) => {
//   const { body, params } = req;
//   const { authorization } = req.headers;
//   const token = authorization ? authorization.split('Bearer ')[1] : false
//   const userId = await getUserIdFromToken(token);

//   const domains = await prisma.updateManyDomains({ where: { id: params.id, user: { id: userId } }, data: body })

//   if (domains.count) {
//     res.status(200).send({ message: "The domain was updated" })
//   } else res.status(500).send({ error: "No domain was found" })
// });

// create an SSL
/**
 * @swagger
 * /domains/{id}/createSSL:
 *   post:
 *     tags:
 *       - "Domains"
 *     summary: "Create an SSL Certificate for this domain name"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        type: "object"
 *        properties:
 *          domain:
 *            type: "string"
 *            example: "domainname.com"
 *          email:
 *            type: "string"
 *            example: "sample@domain.com"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "The ssl was created"
 */
domains.post('/:id/createSSL', async (req, res) => {
  try {
    const { params, body } = req;
    const domainId = params.id;

    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false

    const userId = await getUserIdFromToken(token);
    const user = await getUserInfoFromToken(token);

    // domain exists for that user?
    const domains = await prisma.domains({ where: { id: domainId, user: { id: userId } } })

    // console.log({ domains })

    if (domains.length) {

      const { name } = _.head(domains);
      const { email } = user;

      const { data } = await prismaClient.mutate({
        mutation: createSSLOnServer,
        variables: { domain: name, email }
      })

      console.log({ data })

      res.status(200).send({
        message: "The SSL Was Created"
      })

    } else res.status(500).send({ error: "No domain was found" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
})

export default domains;
