import _ from 'lodash'
import express from 'express'
import { prisma } from "@sc/api/db/prisma/generated/prisma-client";
import { getUserIdFromToken, getUserInfoFromToken } from '../sessions'

const integrations = express.Router()

// get integrations
/**
 * @swagger
 * /integrations:
 *   get:
 *     tags:
 *       - "Integrations"
 *     summary: "Get a list of all the users established for the active user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *              data:
 *                  type: "array"
 *                  items: 
 *                    $ref: "#definitions/Integration"
 */
integrations.get('/', async (req, res) => {
  try {
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token)

    const response = await prisma.integrations({ where: { user: { id: userId } } });
    res.status(200).send({ data: response })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// create new integration
/**
 * @swagger
 * /integrations:
 *   post:
 *     tags:
 *       - "Integrations"
 *     summary: "Create a new integration for the user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/Integration"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          $ref: "#definitions/Integration"
 */
integrations.post('/', async (req, res) => {
  try {
    const { authorization } = req.headers;
    const { body } = req;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token)

    const { setting, type } = body;
    let data = {};

    if (setting) data.setting = setting;
    if (userId) data.user = { connect: { id: userId } }
    if (type) data.type = type;

    const response = await prisma.createIntegration(data);
    res.status(200).send(response)
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// get integration
/**
 * @swagger
 * /integrations/{id}:
 *   get:
 *     tags:
 *       - "Integrations"
 *     summary: "Get details about a specific integration"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *              data:
 *                $ref: "#definitions/Integration"
 */
integrations.get('/:id', async (req, res) => {
  try {
    const { params } = req;
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token);

    const response = await prisma.integrations({ where: { id: params.id, user: { id: userId } } });

    if (response.length) res.status(200).send({ data: _.head(response) })
    else res.status(500).send({ error: "No integration was found" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// delete an integration
/**
 * @swagger
 * /integrations/{id}:
 *   delete:
 *     tags:
 *       - "Integrations"
 *     summary: "Remove an integration profile for a user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "The integration was deleted"
 */
integrations.delete('/:id', async (req, res) => {
  try {
    const { params } = req;
    const integrationId = params.id;
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token);

    const response = await prisma.integrations({ where: { id: params.id, user: { id: userId } } });

    if (response.length) {
      await prisma.deleteIntegration({ id: integrationId })
      res.status(200).send({ message: "The integration was deleted" })
    }
    else res.status(500).send({ error: "No integration was found" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// update new integration
/**
 * @swagger
 * /integrations/{id}:
 *   patch:
 *     tags:
 *       - "Integrations"
 *     summary: "Update information about an integration"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/Integration"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "No integration was found"
 */
integrations.patch('/:id', async (req, res) => {
  try {
    const { body, params } = req;
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token);

    const integrations = await prisma.updateManyIntegrations({ where: { id: params.id, user: { id: userId } }, data: body })

    if (integrations.count) {
      res.status(200).send({ message: "The integration was updated" })
    } else res.status(500).send({ error: "No integration was found" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

export default integrations;
