import _ from 'lodash'
import express from 'express'
import { prisma } from "@sc/api/db/prisma/generated/prisma-client";

import { prismaClient, client } from '../client'
import { getUserIdFromToken, getUserInfoFromToken } from '../sessions'

import { publishPage, makeThisMyHomePage, makeThisMyErrorPage, getPage } from './gql'

const pages = express.Router()

// get pages
/**
 * @swagger
 * /pages:
 *   get:
 *     tags:
 *       - "Pages"
 *     summary: "Get a list of all the pages that the logged in user has created"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *              data:
 *                  type: "array"
 *                  items: 
 *                      $ref: "#definitions/Page"
 */
pages.get('/', async (req, res) => {
    try {
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token)

        const response = await prisma.pages({ where: { user: { id: userId } } });
        res.status(200).send({ data: response.map(({ id, name, updatedAt, slug, createdAt, type, deleted }) => ({ id, name, updatedAt, slug, createdAt, type, deleted })) })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

// create new page
/**
 * @swagger
 * /pages:
 *   post:
 *     tags:
 *       - "Pages"
 *     summary: "Create a new page for the user"
 *     description: "Must be associated with an existing campaign.  Content must include a type = 'BODY' page object"
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/Page"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          $ref: "#definitions/Page"
 */
pages.post('/', async (req, res) => {
    try {
        const { authorization } = req.headers;
        let { body } = req;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token)

        const objectId = _.get(body, 'objectId', false);
        delete body.objectId;

        let data = body;
        if (userId) data.user = { connect: { id: userId } }
        if (objectId) data.object = { connect: { id: objectId } }
        // if (_.has(data, 'content')) data.content = JSON.stringify(data.content)

        console.log({ data })

        const response = await prisma.createPage(data);
        res.status(200).send(response)
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

// get page
/**
 * @swagger
 * /pages/{id}:
 *   get:
 *     tags:
 *       - "Pages"
 *     summary: "Get details about a specific page"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *              data:
 *                $ref: "#definitions/Page"
 */
pages.get('/:id', async (req, res) => {
    try {
        const { authorization } = req.headers;
        const { body, params } = req;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token);

        const response = await prisma.pages({ where: { id: params.id, user: { id: userId } } });
        if (response.length) res.status(200).send({ data: _.head(response) })
        else res.status(500).send({ error: "No page was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

// delete a page
/**
 * @swagger
 * /pages/{id}:
 *   delete:
 *     tags:
 *       - "Pages"
 *     summary: "Remove a page"
 *     description: "(removes from the associated campaign)"
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "The page was deleted"
 */
pages.delete('/:id', async (req, res) => {
    try {
        const { authorization } = req.headers;
        const { params } = req;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token);

        const pages = await prisma.updateManyPages({ where: { id: params.id, user: { id: userId } }, data: { deleted: true } });
        const objects = await prisma.updateManyObjects({ where: { page: { id: params.id } }, data: { deleted: true } });

        if (pages.count) res.status(200).send({ message: "The page was deleted" })
        else res.status(500).send({ error: "No page was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

// update new page
/**
 * @swagger
 * /pages/{id}:
 *   patch:
 *     tags:
 *       - "Pages"
 *     summary: "Update information about a page"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/Page"
 *     responses:
 *       "200":
 *         description: ""
 *       schema:
 *         type: "object"
 *         properties:
 *           message:
 *             type: "string"
 *             example: "The page was updated"
 */
pages.patch('/:id', async (req, res) => {
    try {
        const { body, params } = req;
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false
        const userId = await getUserIdFromToken(token);

        const objectId = _.get(body, 'objectId', false);
        delete body.objectId;

        let data = body;
        if (objectId) data.object = { connect: { id: objectId } }

        const pages = await prisma.updateManyPages({ where: { id: params.id, user: { id: userId } }, data });
        if (pages.count) res.status(200).send({ message: "The page was updated" })
        else res.status(500).send({ error: "No page was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
});

// publish a page
/**
 * @swagger
 * /pages/{id}/publish:
 *   post:
 *     tags:
 *       - "Pages"
 *     summary: "Publish a page"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            webSiteUrl: 
 *              type: "string"
 *              example: "http://subdomain.tenminutefunnels.com"
 *            message: 
 *              type: "string"
 *            statusCode: 
 *              type: "integer"
 *              example: "200"
 */
pages.post('/:id/publish', async (req, res) => {
    try {
        const { params, body } = req;
        const pageId = params.id;
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false

        const userId = await getUserIdFromToken(token);
        const user = await getUserInfoFromToken(token);

        // page exists for that user?
        // prismaClient.query({ fetchPolicy})
        const { data } = await prismaClient.query({
            query: getPage,
            variables: {
                pageId,
                userId
            },
            fetchPolicy: "network-only"
        })

        const { pages } = data;

        const getNextPage = (page) => {
            const { object } = page;
            const { connectTo } = object;

            if (object) {
                const nextPageObj = connectTo ? connectTo.filter((obj) => _.has(obj, "page")) : false;
                if (nextPageObj.length) return _.get(_.head(nextPageObj), 'page.slug', '#');
            }

            return "#";
        }

        if (pages.length) {
            const page = _.head(pages);
            // const page = await prisma.page({ id: pageId });
            let data = {};

            const { appId } = user;

            const pageSettings = _.get(page, 'pageSettings', {});

            const seoPayload = JSON.stringify(_.get(pageSettings, 'seo', {}));
            const emailPayload = JSON.stringify(_.get(pageSettings, 'email', {}));
            const merchantPayload = JSON.stringify(_.get(pageSettings, 'merchant', {}));
            const trackingPayload = JSON.stringify(_.get(pageSettings, 'tracking', {}));
            const cssPayload = _.get(pageSettings, 'css', "").replace(/\n/g, "").replace(/\t/g, "").replace(/ /g, "");
            const exitPayload = JSON.stringify(_.get(pageSettings, 'exitPopup', {}));

            const launchDate = new Date().toISOString();

            // console.log({ appId })
            const { defaultDomain, cloudfrontDistribution } = await prisma.app({ id: appId });

            const { type, theme, slug, content, id } = page;
            const { subdomain } = user;
            // console.log("Content Length", content.length);
            const parentDomain = `${subdomain}.${defaultDomain}`;
            const nextPage = getNextPage(page);

            const isHomePage = _.get(page, 'homePageForDomain.id', '') === id;
            const is404 = _.get(page, 'errorPageForDomain.id', '') === id;

            data = { // required fields
                slug,
                content,
                trackingPayload,
                seoPayload,
                emailPayload,
                merchantPayload,
                cssPayload,
                exitPayload,
                type,
                launchDate,
                nextPage,
                parentDomain,
                subdomain,
                isHomePage,
                is404,
                cloudfrontDistribution,
                ...body,
            };

            if (_.has(data, 'content')) data.content = JSON.stringify(data.content)

            const variables = { ...data, pageId }
            // console.log({ variables })
            const response = await prismaClient.mutate({
                mutation: publishPage,
                variables
            });

            res.status(200).send(response);

        } else res.status(500).send({ error: "No page was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
})

// make a page a home page
/**
 * @swagger
 * /pages/{id}/makeHomePage:
 *   post:
 *     tags:
 *       - "Pages"
 *     summary: "Make this page the default home page for the campaign"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "The page is now the home page"
 */
pages.post('/:id/makeHomePage', async (req, res) => {
    try {
        const { params, body } = req;
        const pageId = params.id;
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false

        const userId = await getUserIdFromToken(token);

        // page exists for that uesr?
        const { data } = await prismaClient.query({
            query: getPage,
            variables: {
                pageId,
                userId
            },
            fetchPolicy: "network-only"
        })

        const { pages } = data;

        if (pages.length) {
            // get domain name
            const domainId = _.get(_.head(pages), 'object.campaign.domain.id');

            const { data } = await prismaClient.mutate({
                mutation: makeThisMyHomePage,
                variables: { pageId: params.id, domainId, ...body }
            });

            res.status(200).send({ message: "The page is now the home page" })
        } else res.status(500).send({ error: "No page was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
})

// make a page a 404 page
/**
 * @swagger
 * /pages/{id}/make404Page:
 *   post:
 *     tags:
 *       - "Pages"
 *     summary: "Make this page the default 404 page for the campaign"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "The page is now the 404 page"
 */
pages.post('/:id/make404Page', async (req, res) => {
    try {
        const { params, body } = req;
        const pageId = params.id;
        const { authorization } = req.headers;
        const token = authorization ? authorization.split('Bearer ')[1] : false

        const userId = await getUserIdFromToken(token);

        // page exists for that uesr?
        const { data } = await prismaClient.query({
            query: getPage,
            variables: {
                pageId,
                userId
            },
            fetchPolicy: "network-only"
        })

        const { pages } = data;

        if (pages.length) {

            // get domain name
            const domainId = _.get(_.head(pages), 'object.campaign.domain.id');

            const { data } = await prismaClient.mutate({
                mutation: makeThisMyErrorPage,
                variables: { pageId: params.id, domainId, ...body }
            })

            res.status(200).send({ message: "The page is now the 404 page" })
        } else res.status(500).send({ error: "No page was found" })
    } catch (err) {
        console.log("ERROR", JSON.stringify(err, null, 2));
        res.status(500).send({
            error: "Sorry, there was an error processing your request",
        })
    }
})

export default pages;
