import _ from "lodash";
import express from "express";
import md5 from "md5";
import redis from "async-redis";
import uuid from 'uuid/v1'
import { getToken } from './gql'

import { client } from '../client'

const redisClient = redis.createClient();

const sessions = express.Router();

const encode = str => Buffer.from(str).toString('base64');
// const decode = str => Buffer.from(str, 'base64').toString('utf-8');

// get sessions
// Routes
/**
 * @swagger
 * /sessions:
 *  get:
 *    tags:
 *      - "Sessions"
 *    summary: "Use to request a new token"
 *    description: "Every api request requires a valid token.  Use this api to receive one."
 *    consumes:
 *    - "application/json"
 *    produces:
 *    - "application/json"
 *    parameters:
 *    - in: "body"
 *      name: "body"
 *      description: "User credentials for logging in to the app"
 *      required: true
 *      schema:
 *        type: "object"
 *        properties:
 *          email:
 *            type: "string"
 *            example: "testing@emailaddress.com"
 *          password:
 *            type: "string"
 *            example: "password"
 *    responses:
 *      "200":
 *        description: "Valid Token"
 *        schema:
 *          type: "object"
 *          properties:
 *            id: 
 *              type: "string"
 *              example: "ck9xdzvr7001q08593jdygmyr"
 *            token:
 *              type: "string"
 *              example: "NzQzNjNiZjAtMjYwNy0xMWViLTkwNzEtMTFlNWZkNzcyZTVm"
 */
sessions.get("/", async (req, res) => {
  try {
    const { email, password } = req.body;

    const result = await client({
      query: getToken,
      variables: { email, password: md5(password) }
    })

    const { data } = result;
    const user = data ? _.head(data.users) : false;

    if (user) {
      const appId = _.get(_.head(user.apps), 'id')
      const host = _.get(_.head(user.apps), 'host')
      const defaultDomain = _.get(_.head(user.apps), 'defaultDomain')
      const cloudfrontDistribution = _.get(_.head(user.apps), 'cloudfrontDistribution')

      delete user.apps;

      if (user.deleted) res.send({ error: "This account has been suspended" });
      else {
        // is there a token for that id?
        const data = await redisClient.get(user.id)

        // if so, delete it
        let token = _.get(JSON.parse(data), 'token', false)
        if (token) await redisClient.del(token)

        // if not, create it
        token = encode(uuid());
        await redisClient.set(token, user.id);
        await redisClient.set(user.id, JSON.stringify({ ...user, token, appId, host, defaultDomain, cloudfrontDistribution }));

        await redisClient.expire(token, 86400); // expire in one day
        await redisClient.expire(user.id, 86400); // expire in one day

        res.send({
          id: user.id,
          token
        })
      }
    }

    else res.send({ error: "Could not authorize using those credentials" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

sessions.get("/:token", async (req, res) => {
  try {
    const { token } = req.params;
    const user = await getUserInfoFromToken(token);
    // console.log({ user, token })
    if (user) res.status(200).send(user)
    else res.status(500).send({ error: "Could not authorize using those credentials" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

sessions.get("/:userId/getToken", async (req, res) => {
  try {
    const { userId } = req.params;
    const token = await getTokenFromUserId(userId);
    if (token) res.status(200).send({ token })
    else res.status(500).send({ error: "Could not find a token by that user" })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
})

/**
 * @swagger
 * /sessions:
 *  delete:
 *     tags:
 *       - "Sessions"
 *     summary: "Use when logging a user out of the api"
 *     description: "A successful request will invalidate the supplied authentication token.  Any attempt to use the invalidated token after a successful DELETE requerst will result in an error"
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *        description: "User was successfully logged out"
 *        schema:
 *          type: "object"
 *          properties:
 *            message:
 *              type: "string"
 *              example: "User has been successfully logged out"
 */
sessions.delete("/", async (req, res) => {
  try {
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false

    await redisClient.del(token)

    res.status(200).send({
      message: "User has been successfully logged out"
    })
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
})

export const checkToken = async (req, res, next) => {
  const { authorization } = req.headers;
  const token = authorization ? authorization.split('Bearer ')[1] : false

  if (token) {
    const userId = await redisClient.get(token)

    if (userId) {
      // renew token expiration times
      await redisClient.expire(token, 86400);
      await redisClient.expire(userId, 86400);
      next()
    } else {
      res.send({ error: "Could not authorize using provided token" })
    }

  } else {
    if (req.path.indexOf('/v1/sessions') > -1) next()
    else if (req.path.indexOf('/v1/users/createUser') > -1) next()
    else if (req.path.indexOf('/v1/iframes') > -1) next()
    else if (req.path.indexOf('/v1/docs') > -1) next()
    else if (req.path === '/') next()

    else res.status(500).send({ error: "Please provide a token" });
  }
}

export const getUserIdFromToken = async (token) => {
  const userId = await redisClient.get(token)
  return userId
}

export const getUserInfoFromToken = async (token) => {

  const userId = await redisClient.get(token)
  const user = await redisClient.get(userId)

  return JSON.parse(user);
}

export const getTokenFromUserId = async (userId) => {
  const user = await redisClient.get(userId)
  try {
    const { token } = JSON.parse(user)
    return token;
  } catch (err) {
    return false;
  }
}

export default sessions;