import _ from "lodash";
import express from "express";
import md5 from "md5";
import fs from 'fs';
import path from "path";
import { prisma } from "@sc/api/db/prisma/generated/prisma-client";
import { prismaClient } from '../client';
import generator from "generate-password";

import { getUserIdFromToken, getUserInfoFromToken } from '../sessions'
import { sendEmail } from "../email";

const curPath = path.join(process.cwd(), "./");

const users = express.Router();

const sendEmailNow = async (data) => {

  // Send Welcome Email  
  const { email } = data;
  const html = fs.readFileSync(`${curPath}users/welcomeEmail.html`, "utf8");
  const parsedEmail = _.template(html)(data);
  const emailResult = await prismaClient.mutate({
    mutation: sendEmail,
    variables: {
      to: email,
      from: "Kyle [10 Minute Funnels] <kylegraham@ambidtech.com>",
      subject: "Welcome to 10 Minute Funnels! (Access Info Inside) :-)",
      text: "",
      html: parsedEmail
    }
  });

  return emailResult;

}

const createUser = async (data, appId) => {
  let { email, firstName, lastName, emailUser = false } = data;
  delete data.emailUser

  const password = _.get(
    data,
    "password",
    generator.generate({ length: 5, numbers: true, uppercase: false })
  );

  const subdomain = _.get(
    data,
    "subdomain",
    `${firstName}${lastName}${Math.random().toFixed(2) * 100}`
  ).toLowerCase();

  // get host from appId
  const { host, name, defaultDomain, cloudfrontDistribution } = await prisma.app({ id: appId })

  const response = await prisma.createUser({
    permissions: [name],
    ...data,
    subdomain,
    password: md5(password),
    domains: {
      create: {
        name: `${subdomain}.${defaultDomain}`, // change to white label domain
        cloudfrontDistribution, // change to white label distribution id
      }
    },
    apps: {
      connect: { id: appId }
    }
  });

  if (emailUser) {
    try {
      await sendEmailNow({ ...data, password });
    } catch (e) {
      console.log(e)
    }
  }

  return response;

};

export const isAdmin = async (userId) => {
  // is the active user an admin account?
  const user = await prisma.user({ id: userId });
  const { permissions } = user;
  return (permissions.indexOf('admin') > -1)
}

// get users
/**
 * @swagger
 * /users:
 *   get:
 *     tags:
 *       - "Users"
 *     summary: "Get a list of all the users belonging to the active user"
 *     description: "This api is only available to administrative accounts"
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *            type: "object"
 *            properties:
 *              data:
 *                  type: "array"
 *                  items:
 *                    $ref: "#definitions/User"
 */
users.get("/", async (req, res) => {
  try {
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token)

    const adminUser = await isAdmin(userId)

    if (adminUser) {
      // show all the users for this white label
      const { appId } = await getUserInfoFromToken(token)
      // console.log({ appId })
      const response = await prisma.users({ where: { apps_some: { id: appId } } })
      res.status(200).send({ data: response });

    } else {
      res.status(500).send({ error: "You do not have permission to see this request" })
    }
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// create new user
/**
 * @swagger
 * /users:
 *   post:
 *     tags:
 *       - "Users"
 *     summary: "Create a new user account"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/User"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          $ref: "#definitions/User"
 */
users.post("/", async (req, res) => {
  try {
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false

    const userId = await getUserIdFromToken(token)
    const { appId } = await getUserInfoFromToken(token)

    const adminUser = await isAdmin(userId)

    if (adminUser) {
      const result = _.has(req, "body.email")
        ? await createUser(req.body, appId)
        : await createUser(req.query, appId);
      res.status(200).send(result);
    } else {
      res.status(500).send({ error: "You do not have permission to see this request" })
    }
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }

});

// simple create
users.post("/createUser", async (req, res) => {
  try {
    // console.log(req.body)
    const { appName } = req.body;
    delete req.body.appName;
    const data = await prisma.apps({ where: { name: appName } });
    if (data.length) {
      const { id } = _.head(data);
      // console.log({ id })
      await createUser(req.body, id)
      res.status(200).send({ message: "New user was created" })
    } else {
      res.status(500).send({ error: "Unable to create the user" })
    }
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
})

// get user
/**
 * @swagger
 * /users/{id}:
 *   get:
 *     tags:
 *       - "Users"
 *     summary: "Get details about a specific user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          type: "object"
 *          properties:
 *              data:
 *                $ref: "#definitions/User"
 */
users.get("/:id", async (req, res) => {
  try {
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token)
    const { appId } = await getUserInfoFromToken(token)
    const adminUser = await isAdmin(userId)

    if (adminUser) {
      const { params } = req;
      const response = await prisma.users({ where: { id: params.id, apps_some: { id: appId } } });
      if (response.length) res.status(200).send({ data: _.head(response) });
      else res.status(500).send({ error: "No user was found" })
    } else {
      res.status(500).send({ error: "You do not have permission to see this request" })
    }
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// delete a user
/**
 * @swagger
 * /users/{id}:
 *   delete:
 *     tags:
 *       - "Users"
 *     summary: "De-activate a user (prevent them from logging in)"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          $ref: "#definitions/User"
 */
users.delete("/:id", async (req, res) => {
  try {
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token)
    const { appId } = await getUserInfoFromToken(token)
    const adminUser = await isAdmin(userId)

    if (adminUser) {
      const { params } = req;
      // user exists?
      const user = await prisma.users({ where: { id: params.id, apps_some: { id: appId } } });
      if (user.length) {
        const response = await prisma.updateUser({
          where: {
            id: params.id,
          },
          data: { deleted: true }
        });
        res.status(200).send(response);
      } else res.status(500).send({ error: "No user was found" })
    } else {
      res.status(500).send({ error: "You do not have permission to see this request" })
    }
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

// update new user
/**
 * @swagger
 * /users/{id}:
 *   patch:
 *     tags:
 *       - "Users"
 *     summary: "Update a user"
 *     description: ""
 *     consumes:
 *     - "application/json"
 *     produces:
 *     - "application/json"
 *     parameters:
 *     - in: "body"
 *       name: "body"
 *       description: ""
 *       schema:
 *        $ref: "#definitions/User"
 *     responses:
 *       "200":
 *         description: ""
 *         schema:
 *          $ref: "#definitions/User"
 */
users.patch("/:id", async (req, res) => {
  try {
    const { authorization } = req.headers;
    const token = authorization ? authorization.split('Bearer ')[1] : false
    const userId = await getUserIdFromToken(token)
    const { appId } = await getUserInfoFromToken(token)
    const adminUser = await isAdmin(userId)

    if (adminUser) {
      let { body, params } = req;
      if (_.has(body, 'password')) body.password = md5(body.password);

      // user exists?
      const user = await prisma.users({ where: { id: params.id, apps_some: { id: appId } } });

      // console.log({ user, body })
      if (user.length) {
        const response = await prisma.updateUser({
          where: { id: params.id },
          data: body
        });
        res.status(200).send(response);
      } else res.status(500).send({ error: "No user was found" })
    } else {
      res.status(500).send({ error: "You do not have permission to see this request" })
    }
  } catch (err) {
    console.log("ERROR", JSON.stringify(err, null, 2));
    res.status(500).send({
      error: "Sorry, there was an error processing your request",
    })
  }
});

export default users;

// resetPassword
// sendResetPasswordEmail
