const express = require("express");
const fs = require("fs-extra");
const bodyParser = require("body-parser");
const shell = require("shelljs");
const _ = require("lodash");
const https = require("https");
const http = require("http");

const homePageDefault = require("./defaults/homePage.json");
const page404Default = require("./defaults/404Page.json");

const getBucket = require("./scripts/getBucket");
const doPageUnPublish = require("./scripts/doPageUnPublish");
const doPageUpdate = require("./scripts/doPageUpdate");
const doQuickPublish = require("./scripts/doQuickPublish");
const generateFontList = require("./scripts/generateFontList");

const sanitize = require("./sanitize");

const app = express();

const settings = require("./config");

const { gatsbySiteLocation, state } = settings;

app.use(bodyParser.json({ limit: "50mb" }));
app.use(bodyParser.urlencoded({ limit: "50mb", extended: true }));

// Setting up server
const httpServer = http.createServer(app);

if (state !== "local") {
  const httpsServer = https.createServer(options, app);
  httpsServer.listen(process.env.PORT || 48080, () => {
    const port = httpsServer.address().port;
    console.log("(Https) App now running on port", port);
  });
}

httpServer.listen(process.env.PORT || 8080, () => {
  const port = httpServer.address().port;
  console.log("(Http) App now running on port", port);
});

// CORS Middleware
app.use((req, res, next) => {
  // Enabling CORS
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Methods", "GET,HEAD,OPTIONS,POST,PUT");
  res.header(
    "Access-Control-Allow-Headers",
    "Origin, X-Requested-With, contentType,Content-Type, Accept, Authorization"
  );
  next();
});

app.get("/", (req, res) => {
  res.send({ message: "Server is running!", statusCode: 200 });
});

app.post("/api/publish", async (req, res) => {
  doPageUpdate(req, res);
});

app.post("/api/unpublish", async (req, res) => {
  doPageUnPublish(req, res);
});

app.post("/api/publishCampaign", (req, res) => {
  // 1. download the content file from the bucket folder
  const body = req.body;
  const contentFilePath = `${gatsbySiteLocation}/data/`;
  const lsCommand = `aws s3 ls s3://${getBucket(body.parentDomain)}/${
    body.parentDomain
    }/content.json`;
  const cpCommand = `aws s3 cp s3://${getBucket(body.parentDomain)}/${
    body.parentDomain
    }/content.json ${contentFilePath}`;

  const publishCampaign = async () => {
    const fileExists = await shell.exec(lsCommand);

    // 2. if none exists, we'll start from scratch
    if (fileExists.code === 1)
      fs.writeFile(`${contentFilePath}content.json`, JSON.stringify([]));
    else await shell.exec(cpCommand);

    // 3. using the posted payload, generate a new content file
    fs.readFile(`${contentFilePath}content.json`, "utf-8", (err, data) => {
      let contentFromFile = JSON.parse(data);
      let existingContent = _.get(contentFromFile, 'pages', []);

      const pageContentArray = JSON.parse(body.objects);

      // console.log("BODY OBJECTS", pageContentArray);
      // console.log("EXISTING CONTENT", existingContent);

      if (pageContentArray.length) {
        // console.log(pageContentArray.length)
        // remove home pages if our payload will be setting one
        const areWeSettingTheHomePage =
          _.findIndex(pageContentArray, pg => pg.isHomePage) > -1;
        if (areWeSettingTheHomePage) {
          existingContent = _.map(existingContent, itm => ({
            ...itm,
            isHomePage: false
          }));
        }

        // remove home pages if our payload will be setting one
        const areWeSettingThe404Page =
          _.findIndex(pageContentArray, pg => pg.is404) > -1;
        if (areWeSettingThe404Page) {
          existingContent = _.map(existingContent, itm => ({
            ...itm,
            is404: false
          }));
        }

        // find the pages on the (existing) s3 server that are NOT part of the active campaign
        const keepThesePages = existingContent.filter(itm => {
          const matchesFound = pageContentArray.filter(
            newItm => newItm.id === itm.id
          );
          return !matchesFound.length;
        });

        let updatedContentObject = [...keepThesePages, ...pageContentArray];

        const pgScaff = {
          seoPayload: "{}",
          emailPayload: "{}",
          merchantPayload: "{}",
          trackingPayload: "{}",
          launchDate: new Date().toISOString(),
          nextPage: "#"
        };

        // are there any 404 pages
        const any404s = _.findIndex(updatedContentObject, pg => pg.is404) > -1;
        // if not add one
        if (!any404s)
          updatedContentObject = [
            ...updatedContentObject,
            {
              ...pgScaff,
              type: "404",
              id: "404",
              slug: "404",
              isHomePage: false,
              is404: true,
              content: JSON.stringify(page404Default)
            }
          ];

        // are there any home pages
        const anyHomePages =
          _.findIndex(updatedContentObject, pg => pg.isHomePage) > -1;

        // if not add one
        if (!anyHomePages)
          updatedContentObject = [
            ...updatedContentObject,
            {
              ...pgScaff,
              type: "home",
              id: "home",
              slug: "home",
              isHomePage: true,
              is404: false,
              content: JSON.stringify(homePageDefault)
            }
          ];

        // console.log("Updated Content Object", updatedContentObject);
        console.log("Any 404s?", any404s);
        console.log("Any Home Pages?", anyHomePages);

        // sanitize
        const sanitizedContent = updatedContentObject.map(obj => {
          const nextPage = obj.hasOwnProperty("nextPage")
            ? obj.nextPage
            : false;
          if (nextPage) {
            return {
              ...obj,
              content: obj.hasOwnProperty("content")
                ? sanitize(obj.content, obj)
                : false,
              trackingPayload: obj.hasOwnProperty("trackingPayload")
                ? sanitize(obj.trackingPayload, obj)
                : false,
            };
          }
          return obj;
        });

        console.log("Sanitized Content", sanitizedContent)

        const updatedConfigWithFontList = generateFontList(
          data,
          sanitizedContent
        );

        console.log("Font List", updatedConfigWithFontList)

        const trackingPayload = body.trackingPayload !== "" ? body.trackingPayload : "{}";

        const updatedContent = {
          scripts: trackingPayload,
          fonts: updatedConfigWithFontList,
          pages: sanitizedContent
        };

        console.log("Updated Content for S3 Upload", updatedContent)

        // 4. now overwrite the current file with my modified file
        fs.writeFile(
          `${contentFilePath}content.json`,
          JSON.stringify(updatedContent),
          error => {
            if (error) {
              console.log("** ERROR **", error);
              return false;
            }

            doQuickPublish(req, res);

          }
        );
      }
    });
  };

  publishCampaign();
});

module.exports = httpServer;
