const path = require("path");
require("dotenv").config({
  path: path.resolve(process.cwd(), "../.env")
});

const express = require("express");
const fs = require("fs-extra");
const bodyParser = require("body-parser");
const shell = require("shelljs");
const _ = require("lodash");
const https = require("https");
const http = require("http");

const homePageDefault = require("./defaults/homePage.json");
const page404Default = require("./defaults/404Page.json");

const sanitize = require("./sanitize");

const app = express();

const settings = require("./config");

const rootLocation = __dirname.substring(0, __dirname.indexOf("sandcastle"));

const { S3_SITES_BUCKET } = process.env;

var options =
  settings.state !== "local"
    ? {
      key: fs.readFileSync(
        `${rootLocation}sandcastle/platform/server/.ssl/privatekey.pem`
      ),
      cert: fs.readFileSync(
        `${rootLocation}sandcastle/platform/server/.ssl/certificate.pem`
      ),
      ca: fs.readFileSync(
        `${rootLocation}sandcastle/platform/server/.ssl/__10minutefunnels_com.ca-bundle`
      )
    }
    : { key: false, cert: false, ca: false };

const gatsbySiteLocation =
  settings.state === "local"
    ? `${rootLocation}sandcastle/apps/gatsby` // <-- local
    : `${rootLocation}sandcastle/apps/gatsby`; // <-- production or staging

// const awsDetails =
//   settings.state === "staging" || settings.state === "local"
//     ? {
//         bucketName: "sandcastlesites-staging",
//         bucketBaseURL:
//           "https://s3-us-west-2.amazonaws.com/sandcastlesites-staging/"
//         // distributionId: "EH53PO6Z5NUP4"
//       }
//     : {
//         bucketName: "sandcastlesites",
//         bucketBaseURL: "https://s3-us-west-2.amazonaws.com/sandcastlesites/"
//         // distributionId: "E31QPSUQXMU2CW"
//       };

const awsDetails = {
  bucketName: S3_SITES_BUCKET,
  bucketBaseURL: `"https://s3-us-west-2.amazonaws.com/${S3_SITES_BUCKET}/`
};

app.use(bodyParser.json({ limit: "50mb" }));
app.use(bodyParser.urlencoded({ limit: "50mb", extended: true }));

// Setting up server
const httpServer = http.createServer(app);

if (settings.state !== "local") {
  const httpsServer = https.createServer(options, app);
  httpsServer.listen(process.env.PORT || 48080, () => {
    const port = httpsServer.address().port;
    console.log("(Https) App now running on port", port);
  });
}

httpServer.listen(process.env.PORT || 8080, () => {
  const port = httpServer.address().port;
  console.log("(Http) App now running on port", port);
});

// CORS Middleware
app.use((req, res, next) => {
  // Enabling CORS
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Methods", "GET,HEAD,OPTIONS,POST,PUT");
  res.header(
    "Access-Control-Allow-Headers",
    "Origin, X-Requested-With, contentType,Content-Type, Accept, Authorization"
  );
  next();
});

app.get("/", (req, res) => {
  res.send({ message: "Server is running!", statusCode: 200 });
});

const generateFontList = (configContent, pageContent) => {
  const content = JSON.parse(pageContent);

  const toProcess = content.map(itm => itm.content);

  // console.log("PAGE CONTENT FOR FONTS", toProcess);

  let fontList = [];

  toProcess.forEach(itm => {
    const itmContent = JSON.parse(itm);
    const fontObjs = itmContent.filter(itm =>
      itm.properties.hasOwnProperty("fontFamily")
    );
    if (fontObjs.length) {
      fontObjs.forEach(itm => {
        const { fontFamily } = itm.properties;
        if (fontList.findIndex(itm => itm === fontFamily))
          fontList.push(fontFamily);
      });
    }
  });

  // console.log("FONT LIST", fontList);

  const returnContent = configContent.replace(
    "<%=fontlist%>",
    _.uniq(fontList)
      .map(l => "`" + l + "`")
      .join(", ")
  );
  // console.log("Return FONT CONTENT", returnContent);

  return returnContent;
};

const getBucket = parentDomain => {
  return parentDomain.indexOf(".tenminutefunnels.com") > -1
    ? "sandcastlesites"
    : awsDetails.bucketName;
};

const doSitePublish = (req, res) => {
  // const contentFilePath = `${gatsbySiteLocation}/data/content.json`;
  const { slug, parentDomain, cloudfrontDistribution } = req.body;

  shell.cd(gatsbySiteLocation);

  const s3uploadCommand = `aws s3 sync ./public/ s3://${getBucket(
    parentDomain
  )}/${parentDomain} --acl public-read-write`;
  console.log("S3 UPLOAD COMANND", s3uploadCommand);

  const cloudfrontInvalidationCommand = `aws cloudfront create-invalidation --distribution-id ${cloudfrontDistribution} --paths "/${parentDomain}/*"`;
  console.log("S3 INVALIDATION COMANND", cloudfrontInvalidationCommand);

  shell.exec(s3uploadCommand, (code, stdout, stderr) => {
    if (stderr)
      return res.status(400).send({
        statusCode: 400,
        status: "Site S3 uploading failed. Please try again",
        message: false
      });
    if (stdout || stdout === "") {
      const foundError = stdout.search("Error:");
      if (foundError > -1)
        return res.status(400).send({
          statusCode: 400,
          status: "There is an error publishing your site",
          message: false
        });
      console.log("WE'RE DONE 1: ");
      // return res.send({ websiteUrl });
    }
    console.log("WE'RE DONE 2: ");

    console.log(cloudfrontInvalidationCommand);

    const webSiteUrl = `${parentDomain}/${slug}`;

    // You should probably run this BEFORE publishing since the invalidation takes a few seconds
    shell.exec(cloudfrontInvalidationCommand, (code, stdout, stderr) => {
      console.log("INVALIDATION COMMAND COMPLETE");

      res.send({
        message: "Site has been published!",
        statusCode: 200,
        data: webSiteUrl
      });
    });
  });
};

const doSiteBuild = (req, res) => {
  const contentFilePath = `${gatsbySiteLocation}/data/`;

  shell.cd(gatsbySiteLocation);

  const shellCommand = "sudo gatsby build";
  console.log("BUILDING SITE: ", shellCommand);

  shell.exec(shellCommand, (code, stdout, stderr) => {
    if (stderr) {
      console.log("*** THERE WAS AN ERROR *** ", stderr);
      console.log("**** > Still Attempting to Publish < ****");
      doSitePublish(req, res);
    }

    if (stdout) {
      const foundError = stdout.search("Error:");
      if (foundError === -1) {
        // copy the content.json file to the public folder
        fs.readFile(`${contentFilePath}content.json`, "utf-8", (err, data) => {
          if (err) console.log("** ERROR READING FILE **", error);
          fs.writeFile(
            `${gatsbySiteLocation}/public/content.json`,
            data,
            error => {
              if (error) console.log("** ERROR WRITING FILE **", error);
              doSitePublish(req, res);
            }
          );
        });
      }
    }
  });
};

const doPageUnPublish = (req, res) => {
  const body = req.body;
  const contentFilePath = `${gatsbySiteLocation}/data/`;

  const cpCommand = `aws s3 cp s3://${getBucket(body.parentDomain)}/${
    body.parentDomain
    }/content.json ${contentFilePath}`;
  shell.exec(cpCommand, (code, stdout) => {
    fs.readFile(`${contentFilePath}content.json`, "utf-8", (err, data) => {
      const existingContent = JSON.parse(data);
      const index = existingContent.findIndex(page => page.id === body.pageId);
      if (index > -1) {
        const updatedContentObject = [
          ...existingContent.slice(0, index),
          ...existingContent.slice(0, index + 1)
        ];

        const updatedContent = JSON.stringify(updatedContentObject);

        // 4. now overwrite the current file with my modified file
        fs.writeFile(
          `${contentFilePath}content.json`,
          updatedContent,
          error => {
            if (error) return false;

            // update path-prefix
            fs.readFile(
              `${gatsbySiteLocation}/gatsby-config-template.js`,
              "utf-8",
              (err, data) => {
                const updatedConfigWithPathPrefix = data.replace(
                  "{pathPrefix}",
                  `${body.parentDomain}`
                );

                fs.writeFile(
                  `${gatsbySiteLocation}/gatsby-config.js`,
                  updatedConfigWithPathPrefix,
                  error => {
                    doSiteBuild(req, res);
                  }
                );
              }
            );
          }
        );
      }
    });
  });
};

const doPageUpdate = (req, res) => {
  // 1. download the content file from the bucket folder
  const body = req.body;
  const contentFilePath = `${gatsbySiteLocation}/data/`;

  const cpCommand = `aws s3 cp s3://${getBucket(body.parentDomain)}/${
    body.parentDomain
    }/content.json ${contentFilePath}`;

  shell.exec(cpCommand, (code, stdout) => {
    // 2. if none exists, we'll start from scratch
    if (!stdout)
      fs.writeFile(`${contentFilePath}content.json`, JSON.stringify([]));

    // 3. using the posted payload, generate a new content file
    fs.readFile(`${contentFilePath}content.json`, "utf-8", (err, data) => {
      const existingContent = JSON.parse(data.trim());
      const index = existingContent.findIndex(page => page.id === body.pageId);

      const obj = {
        id: body.pageId,
        slug: body.slug,
        subdomain: body.subdomain,
        launchDate: body.launchDate,
        content: sanitize(body.content, body),
        seoPayload: body.seoPayload,
        emailPayload: body.emailPayload,
        merchantPayload: body.merchantPayload,
        trackingPayload: sanitize(body.trackingPayload, body),
        nextPage: body.nextPage,
        type: body.type,
        isHomePage: body.isHomePage,
        is404: body.is404
      };

      let updatedContentObject =
        index > -1
          ? [
            ...existingContent.slice(0, index),
            obj,
            ...existingContent.slice(index + 1)
          ]
          : [...existingContent, obj];

      const pgScaff = {
        seoPayload: "{}",
        emailPayload: "{}",
        merchantPayload: "{}",
        trackingPayload: "{}",
        launchDate: new Date().toISOString(),
        nextPage: "#"
      };

      // are there any 404 pages
      const any404s = _.findIndex(updatedContentObject, pg => pg.is404) > -1;
      // if not add one
      if (!any404s)
        updatedContentObject = [
          ...updatedContentObject,
          {
            ...pgScaff,
            type: "404",
            id: "404",
            slug: "404",
            isHomePage: false,
            is404: true,
            content: JSON.stringify(page404Default)
          }
        ];

      // are there any home pages
      const anyHomePages =
        _.findIndex(updatedContentObject, pg => pg.isHomePage) > -1;
      // if not add one
      if (!anyHomePages)
        updatedContentObject = [
          ...updatedContentObject,
          {
            ...pgScaff,
            type: "home",
            id: "home",
            slug: "home",
            isHomePage: true,
            is404: false,
            content: JSON.stringify(homePageDefault)
          }
        ];

      // console.log("Updated Content Object", updatedContentObject);
      // console.log("Any 404s?", any404s);
      // console.log("Any Home Pages?", anyHomePages);

      const updatedContent = JSON.stringify(updatedContentObject);

      // 4. now overwrite the current file with my modified file
      fs.writeFile(`${contentFilePath}content.json`, updatedContent, error => {
        if (error) return false;

        // update path-prefix
        fs.readFile(
          `${gatsbySiteLocation}/gatsby-config-template.js`,
          "utf-8",
          (err, data) => {
            const updatedConfigWithFontList = generateFontList(
              data,
              updatedContent
            );

            // still necessary?
            const updatedConfigWithTrackingScripts = updatedConfigWithFontList.replace(
              "<%=trackingScripts%>",
              "{}"
            );

            const updatedGatsbyConfigFileContent = updatedConfigWithTrackingScripts;

            fs.writeFile(
              `${gatsbySiteLocation}/gatsby-config.js`,
              updatedGatsbyConfigFileContent,
              error => {
                doSiteBuild(req, res);
              }
            );
          }
        );
      });
    });
  });
};

app.post("/api/publish", async (req, res) => {
  doPageUpdate(req, res);
});

app.post("/api/unpublish", async (req, res) => {
  doPageUnPublish(req, res);
});

app.post("/api/publishCampaign", (req, res) => {
  // 1. download the content file from the bucket folder
  const body = req.body;
  const contentFilePath = `${gatsbySiteLocation}/data/`;
  const lsCommand = `aws s3 ls s3://${getBucket(body.parentDomain)}/${
    body.parentDomain
    }/content.json`;
  const cpCommand = `aws s3 cp s3://${getBucket(body.parentDomain)}/${
    body.parentDomain
    }/content.json ${contentFilePath}`;

  const publishCampaign = async () => {
    const fileExists = await shell.exec(lsCommand);

    // 2. if none exists, we'll start from scratch
    if (fileExists.code === 1)
      fs.writeFile(`${contentFilePath}content.json`, JSON.stringify([]));
    else await shell.exec(cpCommand);

    // 3. using the posted payload, generate a new content file
    fs.readFile(`${contentFilePath}content.json`, "utf-8", (err, data) => {
      let existingContent = JSON.parse(data);

      const pageContentArray = JSON.parse(body.objects);

      console.log("BODY OBJECTS", pageContentArray);
      console.log("EXISTING CONTENT", existingContent);

      if (pageContentArray.length) {
        // remove home pages if our payload will be setting one
        const areWeSettingTheHomePage =
          _.findIndex(pageContentArray, pg => pg.isHomePage) > -1;
        if (areWeSettingTheHomePage) {
          existingContent = _.map(existingContent, itm => ({
            ...itm,
            isHomePage: false
          }));
        }

        // remove home pages if our payload will be setting one
        const areWeSettingThe404Page =
          _.findIndex(pageContentArray, pg => pg.is404) > -1;
        if (areWeSettingThe404Page) {
          existingContent = _.map(existingContent, itm => ({
            ...itm,
            is404: false
          }));
        }

        // find the pages on the (existing) s3 server that are NOT part of the active campaign
        const keepThesePages = existingContent.filter(itm => {
          const matchesFound = pageContentArray.filter(
            newItm => newItm.id === itm.id
          );
          return !matchesFound.length;
        });

        let updatedContentObject = [...keepThesePages, ...pageContentArray];

        const pgScaff = {
          seoPayload: "{}",
          emailPayload: "{}",
          merchantPayload: "{}",
          trackingPayload: "{}",
          launchDate: new Date().toISOString(),
          nextPage: "#"
        };

        // are there any 404 pages
        const any404s = _.findIndex(updatedContentObject, pg => pg.is404) > -1;
        // if not add one
        if (!any404s)
          updatedContentObject = [
            ...updatedContentObject,
            {
              ...pgScaff,
              type: "404",
              id: "404",
              slug: "404",
              isHomePage: false,
              is404: true,
              content: JSON.stringify(page404Default)
            }
          ];

        // are there any home pages
        const anyHomePages =
          _.findIndex(updatedContentObject, pg => pg.isHomePage) > -1;

        // if not add one
        if (!anyHomePages)
          updatedContentObject = [
            ...updatedContentObject,
            {
              ...pgScaff,
              type: "home",
              id: "home",
              slug: "home",
              isHomePage: true,
              is404: false,
              content: JSON.stringify(homePageDefault)
            }
          ];

        console.log("Updated Content Object", updatedContentObject);
        console.log("Any 404s?", any404s);
        console.log("Any Home Pages?", anyHomePages);

        // sanitize
        const sanitizedContent = updatedContentObject.map(obj => {
          const nextPage = obj.hasOwnProperty("nextPage")
            ? obj.nextPage
            : false;
          if (nextPage) {
            return {
              ...obj,
              content: obj.hasOwnProperty("content")
                ? sanitize(obj.content, obj)
                : false,
              trackingPayload: obj.hasOwnProperty("trackingPayload")
                ? sanitize(obj.trackingPayload, obj)
                : false
            };
          }
          return obj;
        });

        const updatedContent = JSON.stringify(sanitizedContent);

        // 4. now overwrite the current file with my modified file
        fs.writeFile(
          `${contentFilePath}content.json`,
          updatedContent,
          error => {
            if (error) {
              console.log("** ERROR **", error);
              return false;
            }

            fs.readFile(
              `${gatsbySiteLocation}/gatsby-config-template.js`,
              "utf-8",
              (err, data) => {
                const updatedConfigWithFontList = generateFontList(
                  data,
                  updatedContent
                );

                const trackingPayload =
                  body.trackingPayload !== "" ? body.trackingPayload : "{}";
                const updatedConfigWithTrackingScripts = updatedConfigWithFontList.replace(
                  "<%=trackingScripts%>",
                  trackingPayload
                );

                const updatedGatsbyConfigFileContent = updatedConfigWithTrackingScripts;

                fs.writeFile(
                  `${gatsbySiteLocation}/gatsby-config.js`,
                  updatedGatsbyConfigFileContent,
                  error => {
                    if (error) {
                      console.log(" ** ERROR ** ", error);
                      return false;
                    }
                    // 5. build and publish the changes
                    doSiteBuild(req, res);
                  }
                );
              }
            );
          }
        );
      }
    });
  };

  publishCampaign();
});

module.exports = httpServer;
