import React from 'react'
import { render, fireEvent, cleanup } from '@testing-library/react'

import {
  DefaultComponent,
  WithCheckboxes,
  LoadingState,
  CustomLoadingState,
  EmptyState,
  CustomEmptyState,
  CardsView,
  TableView,
  WithSorting,
  WithCustomFilterComponent,
  WithCustomRowComponent,
  WithCustomBulkSelectedActionsComponent,
  // WithBulkActions,
  WithBulkSelectedActions,
  WithTags,
  WithFilters,
  WithPagination,
  WithAlphabetControls,
  sampleDisplayItemData,
  sampleFieldData,
  // sampleData
} from './DisplayItems.stories'

describe("<DisplayItems /> Rendering", () => {

  afterEach(cleanup)

  it(`Should render component in the dom`, () => {
    const { getByTestId } = render(<DefaultComponent />)
    expect(getByTestId('DisplayItems-Default')).not.toBe(null)
  })

  it(`Should render the same number of records as the amount of items found in the data prop`, () => {
    const { data } = sampleDisplayItemData;
    const { queryAllByTestId } = render(<DefaultComponent />)
    expect(queryAllByTestId('DisplayItems-Card').length).toEqual(data.length)
  })

  it(`Should render a custom row card in the dom (instead of the default) when one is specified in dataItemComponent prop`, () => {
    const { data } = sampleDisplayItemData;
    const defaultComponent = render(<WithCustomRowComponent />)
    expect(defaultComponent.queryAllByTestId('DisplayItems-CustomRowComponent').length).toEqual(data.length)

    cleanup()

    const withCustomRowComponent = render(<DefaultComponent />)
    expect(withCustomRowComponent.queryAllByTestId('DisplayItems-DefaultRowComponent').length).toEqual(data.length)

  })

  it(`Should render the default filter component in dom`, () => {
    const { getByTestId } = render(<WithFilters />)
    expect(getByTestId('FilterGroup-Default')).not.toBe(null)
  })

  it(`Should render a custom filter component in dom when specified in filterComponent prop`, () => {
    const withFilters = render(<WithCustomFilterComponent />)
    expect(withFilters.getByTestId('DisplayItems-CustomFilterComponent')).not.toBe(null)

    cleanup()

    const defaultComponent = render(<WithFilters />)
    expect(defaultComponent.getByTestId('FilterGroup-Default')).not.toBe(null)

  })

  it(`Should not render any filter components when showFilterControls prop is false`, () => {
    const { queryByTestId } = render(<DefaultComponent showFilterControls={false} />)
    expect(queryByTestId('FilterGroup-Default')).toBe(null)
  })

  it(`Should render the AlphabetFilter component when showAlphabetFilter prop is true`, () => {
    const { getByTestId } = render(<WithAlphabetControls />)
    expect(getByTestId('AlphabetFilter-Default')).not.toBe(null)
  })

  it(`Should render any checkboxes (in cards or table) when the showCheckboxes prop is true`, () => {
    const { data } = sampleDisplayItemData
    const { queryAllByTestId } = render(<WithCheckboxes />)
    expect(queryAllByTestId('DisplayItems-Checkbox').length).toEqual(data.length)
  })

  it(`Should render default bulk selected action component in dom when items are selected`, () => {
    // Render the Items with checkboxes
    const { getByTestId, queryAllByTestId } = render(<WithBulkSelectedActions />)

    // Fire click events on one of the checkboxes
    fireEvent.click(queryAllByTestId('DisplayItems-Checkbox')[0].querySelector('input[type="checkbox"]'))

    // Confirm that the bulk selected actions container is present in the dom
    expect(getByTestId('BulkSelectedActions-Default')).not.toBe(null)
  })

  it(`Should render a custom bulk selected action component in dom when one is provided in the bulkActionsComponent prop`, () => {
    // Render the Items with checkboxes
    const { getByTestId, queryAllByTestId } = render(<WithCustomBulkSelectedActionsComponent />)

    // Fire click events on one of the checkboxes
    fireEvent.click(queryAllByTestId('DisplayItems-Checkbox')[0].querySelector('input[type="checkbox"]'))

    // Confirm that the bulk selected actions container is present in the dom
    expect(getByTestId('BulkSelectedActions-Custom')).not.toBe(null)
  })

  it(`Should not render any bulk action components when showBulkActions prop is false`, () => {
    const { queryByTestId } = render(<DefaultComponent showBulkActions={false} />)
    expect(queryByTestId('BulkSelectedActions-Default')).toBe(null)
  })

  it(`Should render a list of saved filter tags if the showTags prop is true and there is data in the tags prop`, () => {
    const { tags } = sampleDisplayItemData
    const { queryAllByTestId } = render(<WithTags />)
    expect(queryAllByTestId('FilterGroup-SavedFilter').length).toEqual(tags.length)
  })

  it(`Should not render any header components when showHeaders prop is false`, () => {
    const withCustomFilterComponent = render(<WithCustomFilterComponent showHeaders={false} />)
    const withFilters = render(<WithFilters showHeaders={false} />)
    expect(withCustomFilterComponent.queryByTestId('DisplayItems-CustomFilterComponent')).toBe(null)
    expect(withFilters.queryByTestId('DisplayItems-DefaultFilterComponent')).toBe(null)
  })

  it(`Should render a sort icons when enableSorting prop is false`, () => {
    const { getByTestId } = render(<WithSorting />)
    expect(getByTestId('DisplayItems-SortingButton')).not.toBe(null)
  })

  it(`Should render pagination controls when showPagination prop is true`, () => {
    const { getByTestId } = render(<WithPagination />)
    expect(getByTestId('DisplayItems-Pagination')).not.toBe(null)
  })

  it(`Should render a default empty state if loading prop is false and there is no data to display`, () => {
    const { getByTestId, queryByTestId } = render(<EmptyState />)
    expect(getByTestId('DisplayItems-EmptyState')).not.toBe(null)
    expect(queryByTestId('DisplayItems-EmptyStateCustom')).toBe(null)
  })

  it(`Should render a custom empty state if loading prop is false, there is no data to display, and a custom empty state is provided`, () => {
    const { getByTestId, queryByTestId } = render(<CustomEmptyState />)
    expect(getByTestId('DisplayItems-EmptyStateCustom')).not.toBe(null)
    expect(queryByTestId('DisplayItems-EmptyState')).toBe(null)
  })

  it(`Should render a default loading image if loading prop is true and no loadingComponent is provided`, () => {
    const { getByTestId, queryByTestId } = render(<LoadingState />)
    expect(getByTestId('DisplayItems-Loading')).not.toBe(null)
    expect(queryByTestId('DisplayItems-LoadingCustom')).toBe(null)
  })

  it(`Should render the loading component found in the loadingComponent prop in the dom if the loading prop is true`, () => {
    const { getByTestId, queryByTestId } = render(<CustomLoadingState />)
    expect(getByTestId('DisplayItems-LoadingCustom')).not.toBe(null)
    expect(queryByTestId('DisplayItems-Loading')).toBe(null)
  })

})

describe("<DisplayItems /> Props", () => {

  afterEach(cleanup)

  it(`Should show a list of cards if type prop is CARD`, () => {
    const { getByTestId } = render(<CardsView />)
    expect(getByTestId('DisplayItems-Cards')).not.toBe(null)
  })

  // it(`Should show a table if type prop is TABLE`, () => {
  //   const { getByTestId } = render(<TableView />)
  //   expect(getByTestId('DisplayItems-Table')).not.toBe(null)
  // })

  // it(`Should show a list of columns in the table that match the columns provided in the components prop`, () => {
  //   const { queryAllByTestId } = render(<TableView />)
  //   expect(queryAllByTestId('DisplayItems-TableColumn').length).toEqual(sampleFieldData.length)
  // })

  it(`Should render the text in title prop as an h2`, () => {
    const { getByTestId } = render(<DefaultComponent />)
    expect(getByTestId('DisplayItems-Title')).not.toBe(null)
  })

  //   it(`Should show a menu of items when bulk action button is clicked that matches the items found in the bulkActions object provided in the props`, () => { })

  //   it(`Should show a menu of items when the 'actions' button on each row/record is clicked that matches items found in the dataItemActions prop`, () => { })
})

describe("<DisplayItems /> Events", () => {

  afterEach(cleanup)

  //   it(`Should trigger onCreate() with appropriate payload when a new record is to be created`, () => { })

  //   it(`Should trigger onUpdate() with appropriate payload when an existing record is updated`, () => { })

  //   it(`Should trigger onDelete() with appropriate payload when the delete option on a record is selected`, () => { })

  //   it(`Should trigger onFilterSave() when the current filter is to be saved`, () => { })

  //   it(`Should trigger onSelect() when an individual record is selected (e.g. clicked)`, () => { })

  //   it(`Should trigger onRowActionSelect() when one of the row action items are selected`, () => { })

  it(`Should trigger onFilterChange() when the filter is to be changed`, () => {
    const onFilterChange = jest.fn()
    const { getByTestId, queryAllByTestId } = render(<WithFilters onFilterChange={onFilterChange} />)

    fireEvent.click(getByTestId('FilterGroup-Button'))

    // Click on one of the fields
    fireEvent.click(queryAllByTestId('FilterGroup-Filter')[0])

    // Click on one of the conditions
    fireEvent.click(getByTestId('FilterGroup-FilterConditions').firstChild.firstChild)

    // Enter a condition in the Filter Condition dialog box
    fireEvent.click(getByTestId('FilterGroup-FilterConditionDialog-OKBtn'))

    // Test conditions
    expect(onFilterChange).toBeCalled()
  })

  //   it(`Should trigger onSort() when a sort is supposed to happen`, () => { })

  //   it(`Should trigger onExport() when an export is supposed to happen`, () => { })

  //   it(`Should trigger onImport() when an import is supposed to happen`, () => { })

  //   it(`Should trigger onBulkActionSelect(action) when one of the bulk actions are selected`, () => { })

  //   it(`Should trigger onPaginationChange() when the page is supposed to change`, () => { })
})

// describe("<DisplayItems /> Methods", () => {

//   afterEach(cleanup)

//   it(`Should trigger goToPage() method when page change occurs`, () => { })

//   it(`Should trigger setRowsPerPage() method when pagination rows per page option changes`, () => { })

//   it(`Should trigger exportData() method when csv download/export option is selected`, () => { })

//   it(`Should select all visible items when selectAll(visible) method is triggered`, () => { })

//   it(`Should select all items that match the existing filter when selectAll(filter) method is triggered`, () => { })

// });

describe("<DisplayItems /> Actions", () => {

  afterEach(cleanup)

  it(`Should perform filter on local data if onChange prop does not return false`, () => {
    console.log = jest.fn()
    const { getByTestId, queryAllByTestId } = render(<WithFilters onFilterChange={(filter) => {
      console.log("Worked")
      return true
    }} />)

    fireEvent.click(getByTestId('FilterGroup-Button'))

    // Click on one of the fields
    fireEvent.click(queryAllByTestId('FilterGroup-Filter')[0])

    // Click on one of the conditions
    fireEvent.click(getByTestId('FilterGroup-FilterConditions').firstChild.firstChild)

    // Enter a condition in the Filter Condition dialog box
    fireEvent.click(getByTestId('FilterGroup-FilterConditionDialog-OKBtn'))

    // Test conditions
    expect(console.log).toBeCalled()
    // expect(onFilterChange).toBeCalled()
  })

  //   it(`Should display total pages in pagination set based on the rowsPerPage property in the paginationSettings prop`, () => { })

  //   it(`Should start record list on the designated page as specified by the 'page' property in the paginationSettings prop`, () => { })
})
