import React from "react";
import { render, fireEvent, cleanup } from "@testing-library/react";
import "@testing-library/jest-dom/extend-expect";
import {
  DefaultRecord,
  WithIcon,
  WithComponent,
  WithTags,
  WithCaption,
  sampleCardData,
} from "./DataCard.stories";
import { DataCard } from '../'
import { bulkActionsSampleData } from '../BulkActionsMenu/BulkActionsMenu.stories'

describe("<DataCard /> Tests", () => {

  afterEach(cleanup)

  it(`Should render the component in the dom`, () => {
    const { getByTestId } = render(<DefaultRecord />)
    expect(getByTestId('DataCard-Default')).not.toBe(null)
  })

  it(`Should show an icon if one is provided in the props`, () => {
    const { getByTestId } = render(<WithIcon />)
    expect(getByTestId('DataCard-Icon')).not.toBe(null)
  })

  it(`Should show a component if one is provided in the component prop`, () => {
    const { getByTestId } = render(<WithComponent />)
    expect(getByTestId('DataCard-Component')).not.toBe(null)
  })

  it(`Should show a list of tags if any are provided in the tags prop`, () => {
    const { getByTestId } = render(<WithTags />)
    expect(getByTestId('DataCard-Tags')).not.toBe(null)
  })

  it(`Should show a caption if one is provided in the label prop`, () => {
    const { getByTestId } = render(<WithCaption />)
    expect(getByTestId('DataCard-Caption')).not.toBe(null)
  })

  it(`Should turn the label caption into an editable text field when clicked if the labelIsEditable prop is set to true`, () => {
    const { getByTestId } = render(<WithCaption />)
    fireEvent.click(getByTestId('DataCard-Caption').firstChild)
    expect(getByTestId('DataCard-CaptionEditing')).not.toBe(null)
  })

  it(`Should trigger the onDataChange event when the textfield is blurred`, () => {
    const onDataChange = jest.fn(itm => itm)
    const { getByTestId } = render(<DataCard label="This is a caption" onDataChange={e => onDataChange(e)} />)
    fireEvent.click(getByTestId('DataCard-Caption').firstChild)
    expect(getByTestId('DataCard-CaptionEditing')).not.toBe(null)
    fireEvent.blur(getByTestId('DataCard-Caption').firstChild)
    expect(onDataChange).toBeCalled()
  })

  it(`Should trigger the onDataChange event when the [enter] key is pressed while in the text field`, () => {
    const onDataChange = jest.fn(itm => itm)
    const { getByTestId } = render(<DataCard label="This is a caption" onDataChange={e => onDataChange(e)} />)
    fireEvent.click(getByTestId('DataCard-Caption').firstChild)
    fireEvent.keyUp(getByTestId('DataCard-Caption').firstChild, { keyCode: 13 })
    expect(getByTestId('DataCard-CaptionLabel')).not.toBe(null)
    expect(onDataChange).toBeCalled()
  })

  it(`Should revert to original text if the [esc] key is pressed while in the text field`, () => {
    const { getByTestId } = render(<DataCard label="This is a caption" onDataChange={e => onDataChange(e)} />)
    fireEvent.click(getByTestId('DataCard-Caption').firstChild)
    fireEvent.keyUp(getByTestId('DataCard-Caption').firstChild, { keyCode: 27 })
    expect(getByTestId('DataCard-CaptionLabel')).not.toBe(null)
  })

  it(`Should revert back to text mode after the onDataChange event has been called`, () => {
    const onDataChange = jest.fn(itm => itm)
    const { getByTestId } = render(<DataCard label="This is a caption" onDataChange={e => onDataChange(e)} />)
    fireEvent.click(getByTestId('DataCard-Caption').firstChild)
    expect(getByTestId('DataCard-CaptionEditing')).not.toBe(null)
    fireEvent.blur(getByTestId('DataCard-Caption').firstChild)
    expect(onDataChange).toBeCalled()
    expect(getByTestId('DataCard-CaptionLabel')).not.toBe(null)
  })

  it(`Should show a dropdown menu (in the form of an icon) if the showActionControls prop is set to true`, () => {
    const { getByTestId } = render(<DefaultRecord />)
    expect(getByTestId('BulkActionsMenu-Icon')).not.toBe(null)
  })

  it(`Should show a list of items in the action controls dropdown menu as specified by the actions prop`, () => {
    const { getByTestId, queryAllByTestId } = render(<DefaultRecord />)
    expect(getByTestId("BulkActionsMenu-Default")).not.toBe(null);
    const myContent = getByTestId("BulkActionsMenu-Icon")
    fireEvent.click(myContent)
    expect(queryAllByTestId('BulkActionsMenu-MenuItem').length).toBe(bulkActionsSampleData.length)
  })

  it(`Should trigger the onActionSelect event when an action item is selected`, () => {
    const onActionSelect = jest.fn(itm => itm)
    const { getByTestId, queryAllByTestId } = render(<DataCard {...sampleCardData} data={bulkActionsSampleData} onActionSelect={itm => onActionSelect(itm)} />);
    const myContent = getByTestId("BulkActionsMenu-Icon")
    fireEvent.click(myContent)
    fireEvent.click(queryAllByTestId('BulkActionsMenu-MenuItem')[0])
    expect(onActionSelect).toBeCalled()
  })

  it(`Should trigger the onClick event when the card is clicked`, () => {
    const onClick = jest.fn(itm => itm)
    const { getByTestId } = render(<DataCard onClick={e => onClick(e)} />)
    fireEvent.click(getByTestId('DataCard-Default'))
    expect(onClick).toBeCalled()
  })

  it(`Should trigger the onDoubleClick event when the card is double-clicked`, () => {
    const onDoubleClick = jest.fn(itm => itm)
    const { getByTestId } = render(<DataCard onDoubleClick={e => onDoubleClick(e)} />)
    fireEvent.doubleClick(getByTestId('DataCard-Default'))
    expect(onDoubleClick).toBeCalled()
  })

});
