import React from 'react'
import { render, fireEvent, cleanup } from '@testing-library/react'

import { DefaultComponent, WithFilters, sampleFilterGroupData, WithSavedFilters, WithDateRange } from './FilterGroup.stories'

describe("<FilterGroup /> Tests", () => {

  afterEach(cleanup)

  it(`Should render in the dom`, () => {
    const { getByTestId } = render(<DefaultComponent />)
    expect(getByTestId('FilterGroup-Default')).not.toBe(null)
  })

  it(`Should show a date range picker when showDateRange in props is true`, () => {
    const { getByTestId } = render(<WithDateRange />)
    expect(getByTestId('FilterGroup-DateRange')).not.toBe(null)
  })

  it(`Should show a list of fields that are available for filtering that corresponds with the fields provided in the fields prop`, () => {
    // Render the component
    const { getByTestId, queryAllByTestId } = render(<WithFilters />)

    // Click the filter button, so that filters open
    fireEvent.click(getByTestId('FilterGroup-Button'))

    // Look for a list of fields to show that match the list of fields passed in the fields prop
    expect(queryAllByTestId('FilterGroup-Filter').length).toEqual(sampleFilterGroupData.fields.length)
  })

  it(`Should show a list of filter conditions in the dom that corresponds with the list provided in the data props`, () => {
    const { getByTestId, queryAllByTestId } = render(<WithFilters />)

    // Click the filter button, so that filters open
    fireEvent.click(getByTestId('FilterGroup-Button'))

    // Click on one of the fields
    fireEvent.click(queryAllByTestId('FilterGroup-Filter')[0])

    // Confirm that the conditions container is showing in the dom
    expect(getByTestId('FilterGroup-FilterConditions')).not.toBe(null)

  })

  it(`Should show a list of savedFilters in the dom that corresponds with the savedFilters prop if the savedFiltersIsEnabled prop is set to true`, () => {

    // Render it
    const { queryAllByTestId } = render(<WithSavedFilters />)

    // Confirm that the saved filters are showing (matches the amount provided)
    expect(queryAllByTestId('FilterGroup-SavedFilter').length).toEqual(sampleFilterGroupData.savedFilters.length)

  })
});

describe("<FilterGroup /> Events", () => {

  afterEach(cleanup)

  it(`Should trigger onChange event when date picker changes`, () => {
    const onChange = jest.fn()
    const { getByTestId } = render(<WithDateRange onChange={onChange} />)
    fireEvent.change(getByTestId('FilterGroup-DateRange').querySelectorAll('input')[0], { target: { value: new Date() } })
    expect(onChange).toBeCalled()
  })

  it(`Should trigger the onChange event whenever a filter is selected`, () => {
    const onChange = jest.fn()
    const { getByTestId, queryAllByTestId } = render(<WithDateRange onChange={onChange} />)

    // Click the filter button, so that filters open
    fireEvent.click(getByTestId('FilterGroup-Button'))

    // Click on one of the fields
    fireEvent.click(queryAllByTestId('FilterGroup-Filter')[0])

    // Click on one of the conditions
    fireEvent.click(getByTestId('FilterGroup-FilterConditions').firstChild.firstChild)

    // Enter a condition in the Filter Condition dialog box
    // fireEvent.change(getByTestId('FilterGroup-FilterConditionDialog').querySelector('input'), { target: { value: 'Sample Text' } })
    fireEvent.click(getByTestId('FilterGroup-FilterConditionDialog-OKBtn'))

    // Test conditions
    expect(onChange).toBeCalled()
  })

  // it(`Should trigger the onCreate event when a new filter is created or added to the group`, () => { })

  // it(`Should trigger the onDelete event when a filter group is deleted`, () => { })

  // it(`Should trigger the onClear event when all the filters are cleared`, () => { })

  // it(`Should trigger the onSave event when the chosen group of filters are saved`, () => { })

});
