import * as React from "react";
import { render, screen, fireEvent, cleanup } from "@testing-library/react";
import { EditorMode } from "@sc/modules/v2/Editor/types";
import reactToCSS from "react-style-object-to-css";
import { styleClean } from "@sc/utils";

import DropdownField from "./DropdownField";
import { styleData } from "../../FormBuilder.stories";
import { IconTypes } from "@sc/plugins/webcomponents/v2/Icon";

const testId = "FormBuilder-FormFields-DropdownField";
const props = {
  data: [],
  styles: styleData,
  attributeContainerDataSettings: { hide: false },
};

describe("<DropdownField />  Rendering Tests", () => {
  it(`Should render in the dom`, () => {
    render(<DropdownField {...props} />);
    expect(screen.queryByTestId(testId)).toBeTruthy();
  });

  it(`Should render an EDITOR or LIVE version of the component based on the value of the mode prop`, () => {
    const testIdLive = "FormBuilder-FormFields-DropdownField-LIVE";
    const testIdEditor = "FormBuilder-FormFields-DropdownField-EDITOR";

    const { rerender } = render(
      <DropdownField {...props} mode={EditorMode.LIVE} />
    );

    expect(screen.queryByTestId(testIdLive)).toBeTruthy();

    rerender(<DropdownField {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(testIdEditor)).toBeTruthy();
  });

  it(`Should render a regular dropdown input field when the mode is live`, () => {
    render(<DropdownField {...props} mode={EditorMode.LIVE} />);
    expect(screen.queryByTestId(testId).querySelector("select")).toBeTruthy();
  });

  it(`Should render the AttributeContainer component when the mode is edit`, () => {
    const attrTestId = "FormBuilder-AttributeContainer";
    render(<DropdownField {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(attrTestId)).toBeTruthy();
  });

  it(`Should see that the <Label /> component has been rendered in the dom`, () => {
    const labelTestId = "FormBuilder-Attributes-Label";
    render(<DropdownField {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(labelTestId)).toBeTruthy();
  });

  it(`Should see that the <IconSelector /> component has been rendered in the dom`, () => {
    render(<DropdownField {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId("Properties-IconSelector-input")).toBeTruthy();
  });

  it(`Should see that the <Description /> component has been rendered in the dom`, () => {
    const descriptionTestId = "FormBuilder-Attributes-Description";
    render(<DropdownField {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(descriptionTestId)).toBeTruthy();
  });

  it(`Should see that the <Column /> component has been rendered in the dom`, () => {
    const columnsTestId = "FormBuilder-Attributes-Columns";
    render(<DropdownField {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(columnsTestId)).toBeTruthy();
  });

  it(`Should see that the <Required /> component has been rendered in the dom`, () => {
    const requiredTestId = "FormBuilder-Attributes-Required";
    render(<DropdownField {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(requiredTestId)).toBeTruthy();
  });

  it(`Should see that the <SortableList /> component has been rendered in the dom`, () => {
    const sortableListTestId = "Properties-SortableList";
    render(<DropdownField {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(sortableListTestId)).toBeTruthy();
  });

  it(`Should render an inputStyle, iconStyle, descriptionStyle, containerStyle, and labelStyle when specified`, () => {
    const label = "Label Check";
    const description = "Description Check";

    const { container } = render(
      <DropdownField
        {...props}
        icon={IconTypes.Email}
        label={label}
        description={description}
      />
    );

    expect(container.firstChild).toMatchSnapshot();

    // // inputStyle
    // expect(
    //   styleClean(
    //     screen
    //       .getByTestId(testId)
    //       .querySelector("select")
    //       .getAttribute("style")
    //   )
    // ).toContain(
    //   styleClean(
    //     reactToCSS({ ...styleData.inputStyle, padding: "15px 15px 15px 40px" })
    //   )
    // );

    // // iconStyle
    // expect(
    //   styleClean(screen.getByTestId("WC-ICON-LIVE").getAttribute("style"))
    // ).toContain(styleClean(reactToCSS(styleData.iconStyle)));

    // // descriptionStyle
    // expect(
    //   styleClean(screen.getByText(description).getAttribute("style"))
    // ).toContain(styleClean(reactToCSS(styleData.descriptionStyle)));

    // // containerStyle
    // expect(
    //   styleClean(screen.getByTestId(testId).getAttribute("style"))
    // ).toContain(styleClean(reactToCSS(styleData.containerStyle)));

    // // labelStyle
    // expect(styleClean(screen.getByText(label).getAttribute("style"))).toContain(
    //   styleClean(reactToCSS(styleData.labelStyle))
    // );
  });

  test.skip(`Should only see that the <Validation /> component has been rendered if the "hide" value is false`, () => {});
});
describe("<DropdownField /> Event Tests", () => {
  test.todo(
    `Should trigger the onChange event (via handleChange()) if the AttributeContainer's onChange event is called`
  );

  test.todo(
    `Should trigger the onChange event with appropriate payload (via handleChange()) when the list sortable items change`
  );
});
describe("<DropdownField /> Action Tests", () => {
  test.todo(
    `handleChange should convert the incoming payload to LiveFormFieldProps`
  );
});
