import * as React from "react";
import { render, screen, fireEvent, cleanup } from "@testing-library/react";

import { EditorMode } from "../../../../Editor/types";
import MultipleChoice from "./MultipleChoice";
import { data } from "./MultipleChoice.stories";

const testId = "FormBuilder-FormFields-MultipleChoice";
const props = { data, attributeContainerDataSettings: { hide: false } };

describe("<MultipleChoice /> Rendering Tests", () => {
  it(`Should render in the dom`, () => {
    render(<MultipleChoice {...props} />);
    expect(screen.queryByTestId(testId)).toBeTruthy();
  });

  it(`Should render an EDITOR or LIVE version of the component based on the value of the mode prop`, () => {
    const testIdLive = "FormBuilder-FormFields-MultipleChoice-LIVE";
    const testIdEditor = "FormBuilder-FormFields-MultipleChoice-EDITOR";

    const { rerender } = render(
      <MultipleChoice {...props} mode={EditorMode.LIVE} />
    );
    expect(screen.queryByTestId(testIdLive)).toBeTruthy();

    rerender(<MultipleChoice {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(testIdEditor)).toBeTruthy();
  });

  it(`Should render a list of multiple choice radio buttons when the mode is live`, () => {
    const testIdLive = "FormBuilder-FormFields-MultipleChoice-LIVE";
    render(<MultipleChoice {...props} data={data} />);
    expect(
      screen.queryByTestId(testIdLive).querySelectorAll("input").length
    ).toEqual(data.length);
  });

  it(`Should render the AttributeContainer component when the mode is edit`, () => {
    const attrTestId = "FormBuilder-AttributeContainer";
    render(<MultipleChoice {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(attrTestId)).toBeTruthy();
  });

  it(`Should see that the <Label /> component has been rendered in the dom`, () => {
    const labelTestId = "FormBuilder-Attributes-Label";
    render(<MultipleChoice {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(labelTestId)).toBeTruthy();
  });

  it(`Should see that the <Description /> component has been rendered in the dom`, () => {
    const descriptionTestId = "FormBuilder-Attributes-Description";
    render(<MultipleChoice {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(descriptionTestId)).toBeTruthy();
  });

  it(`Should see that the <Column /> component has been rendered in the dom`, () => {
    const columnsTestId = "FormBuilder-Attributes-Columns";
    render(<MultipleChoice {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(columnsTestId)).toBeTruthy();
  });

  it(`Should see that the <Required /> component has been rendered in the dom`, () => {
    const requiredTestId = "FormBuilder-Attributes-Required";
    render(<MultipleChoice {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(requiredTestId)).toBeTruthy();
  });

  it(`Should see that the <SortableList /> component has been rendered in the dom`, () => {
    const sortableListTestId = "Properties-SortableList";
    render(<MultipleChoice {...props} mode={EditorMode.EDITOR} />);
    expect(screen.queryByTestId(sortableListTestId)).toBeTruthy();
  });

  test.skip(`Should only see that the <Validation /> component has been rendered if the "hide" value is false`, () => {});
});

describe("<MultipleChoice /> Event Tests", () => {
  test.todo(
    `Should trigger the onChange event (via handleChange()) if the AttributeContainer's onChange event is called`
  );

  it(`Should trigger the onChange event with appropriate payload (via handleChange()) when the list sortable items change`, () => {
    const onChange = jest.fn();
    render(
      <MultipleChoice {...props} onChange={onChange} mode={EditorMode.LIVE} />
    );
    fireEvent.click(
      screen
        .queryAllByTestId(
          "FormBuilder-FormFields-MultipleChoice-RadioButton"
        )[0]
        .querySelector("input"),
      { checked: true }
    );
    expect(onChange).toBeCalled();
  });
});

describe("<MultipleChoice /> Actions Tests", () => {
  test.todo(
    `handleChange should convert the incoming payload to LiveFormFieldProps`
  );
});
