import * as React from "react";
import { render, screen, fireEvent, cleanup } from "@testing-library/react";

import "@babel/polyfill";
import "@testing-library/jest-dom/extend-expect";
import {
  mockGetComputedSpacing,
  mockDndElSpacing,
  makeDnd,
  DND_DRAGGABLE_DATA_ATTR,
  DND_DIRECTION_DOWN,
  DND_DIRECTION_UP,
} from "react-beautiful-dnd-test-utils";

import SortableList from "./SortableList";

import { ItemComponentProps } from "./types";
import { propsData } from "./SortableList.stories";

describe("<SortableList /> Rendering", () => {
  afterEach(cleanup);

  it(`Should render in the dom`, () => {
    render(<SortableList {...propsData} />);
    expect(screen.queryByTestId(`Properties-SortableList`)).toBeTruthy();
  });

  it(`Should render the same number of item components as the number of items in the provided data prop`, () => {
    const data: ItemComponentProps[] = [{ id: "1" }, { id: "2" }, { id: "3" }];
    const testId = "Properties-SortableList-ItemComponent";

    render(
      <SortableList
        {...propsData}
        data={data}
        ItemComponent={(props) => <div data-testid={testId} />}
      />
    );
    expect(screen.queryAllByTestId(testId).length).toBe(data.length);
  });

  it(`Should pass along the object's payload as props to the ItemComponent`, () => {
    // Declare an id and pass it through the props of the ItemComponent component
    // Make the ItemComponent component show it as a data-testid and verify that you can see it on render

    const id = "ThisIsATest";

    interface iProps {
      id: string;
    }

    const MyTestComponent: React.FC<iProps> = (props) => (
      <div data-testid={props.id} />
    );

    render(
      <SortableList
        {...propsData}
        data={[{ id }]}
        ItemComponent={MyTestComponent}
      />
    );

    expect(screen.queryByTestId(id)).toBeTruthy();
  });

  it(`Should render an AddButtonComponent (instead of the default component) in each item if one is provided`, () => {
    const testId = "Properties-SortableList-AddButtonComponent";
    render(
      <SortableList
        {...propsData}
        AddButtonComponent={(props) => <div data-testid={testId} />}
      />
    );
    expect(screen.queryByTestId(testId)).toBeTruthy();
  });

  it(`Should render an EditButtonComponent (instead of the default component) in each item if one is provided`, () => {
    const testId = "Properties-SortableList-EditButtonComponent";
    render(
      <SortableList
        {...propsData}
        EditButtonComponent={(props) => <div data-testid={testId} />}
      />
    );
    expect(screen.queryAllByTestId(testId).length).toEqual(
      propsData.data.length
    );
  });

  it(`Should render an DeleteButtonComponent (instead of the default component) in each item if one is provided`, () => {
    const testId = "Properties-SortableList-DeleteButtonComponent";
    render(
      <SortableList
        {...propsData}
        DeleteButtonComponent={(props) => <div data-testid={testId} />}
      />
    );
    expect(screen.queryAllByTestId(testId).length).toEqual(
      propsData.data.length
    );
  });

  it(`Should not display a delete button if the canDelete prop is false`, () => {
    const testId = "Properties-SortableList-DefaultDeleteButton";
    const { data } = propsData;
    const { rerender } = render(<SortableList {...propsData} canDelete />);
    expect(screen.queryAllByTestId(testId).length).toEqual(data.length);

    rerender(<SortableList {...propsData} canDelete={false} />);
    expect(screen.queryAllByTestId(testId).length).toEqual(0);
  });

  it(`Should not display an add button if the canAdd prop is false`, () => {
    const testId = "Properties-SortableList-DefaultAddButton";
    const { rerender } = render(<SortableList {...propsData} canAdd />);
    expect(screen.queryByTestId(testId)).toBeTruthy();

    rerender(<SortableList {...propsData} canAdd={false} />);
    expect(screen.queryAllByTestId(testId).length).toEqual(0);
  });

  it(`Should not display an edit button if the canEdit prop is false`, () => {
    const testId = "Properties-SortableList-DefaultEditButton";
    const { data } = propsData;
    const { rerender } = render(<SortableList {...propsData} canEdit />);
    expect(screen.queryAllByTestId(testId).length).toEqual(data.length);

    rerender(<SortableList {...propsData} canEdit={false} />);
    expect(screen.queryAllByTestId(testId).length).toEqual(0);
  });

  it(`Should show a drag handle icon if the showDragHandle prop is true`, () => {
    const testId = "Properties-SortableList-DragHandle";
    const { data } = propsData;
    render(<SortableList {...propsData} showCRUDOnHover={false} />);
    expect(screen.queryAllByTestId(testId).length).toEqual(data.length);
  });
});

describe("<SortableList /> Actions", () => {
  afterEach(cleanup);

  beforeEach(() => {
    mockGetComputedSpacing();
  });

  it(`Should display CRUD icons only when mouse is hovering on an item if showCRUDOnHover is true, otherwise always display`, () => {
    const testId1 = "Properties-SortableList-Item";
    const testId2 = "Properties-SortableList-DragHandle";
    const itm = 1;

    const { data } = propsData;

    // Show the element with showCrudOnHover = false -- aka, always show crud
    const { rerender } = render(
      <SortableList {...propsData} showCRUDOnHover={false} />
    );

    expect(screen.queryAllByTestId(testId2).length).toEqual(data.length);

    // Show the element with showCrudOnHover = true -- aka, dont' show unless hovered
    rerender(<SortableList {...propsData} showCRUDOnHover={true} />);
    expect(screen.queryByTestId(testId2)).toBeNull();

    // Trigger the mouse hover
    fireEvent.mouseEnter(screen.queryAllByTestId(testId1)[itm]);
    expect(screen.queryByTestId(testId2)).toBeTruthy();
  });

  it(`Should not allow drag and drop actions if the canDragAndDrop prop is false`, async () => {
    const onChange = jest.fn();

    const data = [
      { id: "1", firstName: "Kyle" },
      { id: "2", firstName: "Linda" },
      { id: "3", firstName: "Mikel" },
      { id: "4", firstName: "Nevaeh" },
    ];

    const { rerender } = render(
      <SortableList
        {...propsData}
        data={data}
        onChange={onChange}
        showCRUDOnHover={false}
        ItemComponent={(itm) => <div>{itm.firstName}</div>}
        canDragAndDrop
      />
    );

    expect(
      screen.getByText(data[0]["firstName"]).closest('[draggable="false"]')
    ).toBeTruthy();

    rerender(
      <SortableList
        {...propsData}
        data={data}
        onChange={onChange}
        showCRUDOnHover={false}
        ItemComponent={(itm) => <div>{itm.firstName}</div>}
        canDragAndDrop={false}
      />
    );

    expect(
      screen.getByText(data[0]["firstName"]).closest('[draggable="false"]')
    ).toBeNull();
  });
});

describe("<SortableList /> Events", () => {
  afterEach(cleanup);

  it(`Should trigger the onChange event when the items are re-ordered via drag-and-drop`, async () => {
    const onChange = jest.fn();

    const data = [
      { id: "1", firstName: "Kyle" },
      { id: "2", firstName: "Linda" },
      { id: "3", firstName: "Mikel" },
      { id: "4", firstName: "Nevaeh" },
    ];

    const rtlUtils = render(
      <SortableList
        {...propsData}
        data={data}
        onChange={onChange}
        showCRUDOnHover={false}
        ItemComponent={(itm) => <div>{itm.firstName}</div>}
      />
    );

    const { getByText } = rtlUtils;
    mockDndElSpacing(rtlUtils);
    const makeGetDragEl = (text) => () =>
      rtlUtils.getByText(text).closest(DND_DRAGGABLE_DATA_ATTR);

    const positions = 2;

    await makeDnd({
      getByText,
      getDragEl: makeGetDragEl(data[0]["firstName"]),
      direction: DND_DIRECTION_DOWN,
      positions,
    });

    expect(onChange).toBeCalledTimes(positions);
  });

  it(`Should include an updated list of items from the onChange event return function when the sections are re-ordered via drag and drop`, async () => {
    const onChange = jest.fn();

    const data = [
      { id: "1", firstName: "Kyle" },
      { id: "2", firstName: "Linda" },
      { id: "3", firstName: "Mikel" },
      { id: "4", firstName: "Nevaeh" },
    ];

    const rtlUtils = render(
      <SortableList
        {...propsData}
        data={data}
        onChange={onChange}
        showCRUDOnHover={false}
        ItemComponent={(itm) => <div>{itm.firstName}</div>}
      />
    );

    const { getByText } = rtlUtils;
    mockDndElSpacing(rtlUtils);
    const makeGetDragEl = (text) => () =>
      rtlUtils.getByText(text).closest(DND_DRAGGABLE_DATA_ATTR);

    await makeDnd({
      getByText,
      getDragEl: makeGetDragEl(data[0]["firstName"]),
      direction: DND_DIRECTION_DOWN,
      positions: 2,
    });

    expect(onChange.mock.calls[0][0][1]).toMatchObject(data[0]);
  });

  it(`Should trigger the onEdit event when an edit button is clicked`, () => {
    // mock onEdit
    const onEdit = jest.fn();

    // render a static component
    render(
      <SortableList {...propsData} onEdit={onEdit} showCRUDOnHover={false} />
    );

    // find the first edit button and click it
    fireEvent.click(
      screen
        .queryAllByTestId("Properties-SortableList-DefaultEditButton")[1]
        .querySelector("button")
    );

    // confirm that onEdit was fired
    expect(onEdit).toBeCalled();
  });

  it(`Should include the associated object's payload when the onEdit event is called`, () => {
    // mock onEdit
    const onEdit = jest.fn();
    const { data } = propsData;

    const itm = 1;

    // render a static component
    render(
      <SortableList {...propsData} onEdit={onEdit} showCRUDOnHover={false} />
    );

    // find the first edit button and click it
    fireEvent.click(
      screen
        .queryAllByTestId("Properties-SortableList-DefaultEditButton")
        [itm].querySelector("button")
    );

    // confirm that onEdit was fired
    expect(onEdit.mock.calls[0][0]).toMatchObject(data[itm]);
  });

  it(`Should trigger the onAdd event when the add button is clicked`, () => {
    // mock onAdd
    const onAdd = jest.fn();

    // render a static component
    render(
      <SortableList {...propsData} onAdd={onAdd} showCRUDOnHover={false} />
    );

    // find the first delete button and click it
    fireEvent.click(
      screen
        .queryByTestId("Properties-SortableList-DefaultAddButton")
        .querySelector("button")
    );

    // confirm that onAdd was fired
    expect(onAdd).toBeCalled();
  });

  it(`Should trigger the onDelete event when a delete button is clicked`, () => {
    // mock onEdit
    const onDelete = jest.fn();

    // render a static component
    render(
      <SortableList
        {...propsData}
        onDelete={onDelete}
        showCRUDOnHover={false}
      />
    );

    // find the first delete button and click it
    fireEvent.click(
      screen
        .queryAllByTestId("Properties-SortableList-DefaultDeleteButton")[1]
        .querySelector("button")
    );

    // confirm that onDelete was fired
    expect(onDelete).toBeCalled();
  });

  it(`Should include the associated object's payload when the onDelete event is called`, () => {
    // mock onDelete
    const onDelete = jest.fn();
    const { data } = propsData;

    const itm = 1;

    // render a static component
    render(
      <SortableList
        {...propsData}
        onDelete={onDelete}
        showCRUDOnHover={false}
      />
    );

    // find the first edit button and click it
    fireEvent.click(
      screen
        .queryAllByTestId("Properties-SortableList-DefaultDeleteButton")
        [itm].querySelector("button")
    );

    // confirm that onDelete was fired
    expect(onDelete.mock.calls[0][0]).toMatchObject(data[itm]);
  });
});
