import * as React from "react";
import { DragDropContext, Droppable, Draggable } from "react-beautiful-dnd";
import styled from "styled-components";

import theme from "@sc/components/ui/theme";

import {
  SortableListProps,
  AddButtonComponentProps,
  EditButtonComponentProps,
  DeleteButtonComponentProps,
} from "./types";
import Icon from "@sc/plugins/webcomponents/v2/Icon";
import { IconTypes } from "@sc/plugins/webcomponents/v2/Icon";

const styles = {
  BUTTON: {
    background: "transparent",
    border: "none",
    cursor: "pointer",
    marginTop: 10,
  },
};

const Placeholder = () => (
  <button style={styles.BUTTON}>
    <Icon type={IconTypes.Cancel} style={{ color: "transparent" }} />
  </button>
);

const StyledButton = styled.button`
  width: 100%;
  padding: 10px;
  margin-top: 20px;
  background-color: white;
  border: 1px solid #ddd;
  border-bottom-width: 2px;
  border-radius: 4px;
  cursor: pointer;
  color: ${theme.primaryColor};
  &:hover {
    background-color: #f4f4f4;
  }
`;

const DefaultAddButton: React.FC<AddButtonComponentProps> = ({
  onAdd,
  name,
}) => (
  <span
    data-testid="Properties-SortableList-DefaultAddButton"
    style={{ display: "flex", alignItems: "center" }}
  >
    {/* <Placeholder /> */}

    <StyledButton onClick={onAdd}>
      <div style={{ display: "flex" }}>
        <span
          style={{
            backgroundColor: "#EEE",
            borderRadius: 3,
            padding: "2px 4px",
            marginRight: 10,
          }}
        >
          <Icon type={IconTypes.Add} />
        </span>
        <div style={{ padding: 5, fontSize: "11pt" }}>Add New {name}</div>
      </div>
    </StyledButton>
    {/* <Placeholder /> */}
    {/* <Placeholder /> */}
  </span>
);

const DefaultEditButton: React.FC<EditButtonComponentProps> = ({
  onEdit,
  payload,
}) => (
  <span data-testid="Properties-SortableList-DefaultEditButton">
    <button onClick={() => onEdit(payload)} style={styles.BUTTON}>
      <Icon type={IconTypes.Edit} />
    </button>
  </span>
);
const DefaultDeleteButton: React.FC<DeleteButtonComponentProps> = ({
  onDelete,
  payload,
}) => (
  <span data-testid="Properties-SortableList-DefaultDeleteButton">
    <button onClick={() => onDelete(payload)} style={styles.BUTTON}>
      <Icon type={IconTypes.Cancel} />
    </button>
  </span>
);

/**
 * A component that provides a nice UX for data entry of lists.
 *
 * Displays a list of items based on the incoming array,
 * facilitates CRUD actions via drag and drop and customizable
 * buttons and display types.
 */
export const SortableList: React.FC<SortableListProps> = ({
  data = [],
  ItemComponent,
  name = "Item",
  AddButtonComponent = DefaultAddButton,
  EditButtonComponent = DefaultEditButton,
  DeleteButtonComponent = DefaultDeleteButton,
  showCRUDOnHover = true,
  showDragHandle = true,
  canDragAndDrop = true,
  canDelete = true,
  canAdd = true,
  canEdit = true,
  showItemInline = true,
  style = {},
  itemStyle = {},
  itemHoverStyle = {},
  onChange = () => null,
  onEdit = () => null,
  onAdd = () => null,
  onDelete = () => null,
}) => {
  const SortableItem: React.FC<any> = (sortableItemProps) => {
    const [isHovering, setIsHovering] = React.useState<boolean>(false);

    const handleDelete = (payload) => {
      // send back the payload as well as a mutated version of the payload
      const { id } = payload;
      const updatedData = data.filter((itm) => itm.id !== id);
      console.log({ payload, data, updatedData });

      onDelete(payload, updatedData);
    };

    return (
      <>
        <div
          onMouseEnter={() => setIsHovering(true)}
          onMouseLeave={() => setIsHovering(false)}
          data-testid="Properties-SortableList-Item"
          style={{
            display: "flex",
            ...itemStyle,
            ...(isHovering ? itemHoverStyle : {}),
          }}
        >
          {(isHovering || !showCRUDOnHover) && showDragHandle ? (
            <span
              data-testid="Properties-SortableList-DragHandle"
              // style={{ position: "absolute", marginLeft: -25 }}
            >
              <button
                data-testid="Properties-SortableList-DragHandle"
                style={{ ...styles.BUTTON, cursor: "move" }}
              >
                <Icon type={IconTypes.DragIndicator} />
              </button>
            </span>
          ) : (
            showDragHandle && <Placeholder />
          )}
          {showItemInline ? (
            <ItemComponent {...sortableItemProps} isHovering={isHovering} />
          ) : (
            <div style={{ width: "100%" }} />
          )}
          {(isHovering || !showCRUDOnHover) && canEdit ? (
            <EditButtonComponent onEdit={onEdit} payload={sortableItemProps} />
          ) : // <Placeholder />
          null}
          {(isHovering || !showCRUDOnHover) && canDelete ? (
            <DeleteButtonComponent
              onDelete={handleDelete}
              payload={sortableItemProps}
            />
          ) : // <Placeholder />
          null}
        </div>
        {!showItemInline && (
          <ItemComponent
            {...sortableItemProps}
            isHovering={isHovering}
            onMouseEnter={() => setIsHovering(true)}
            onMouseLeave={() => setIsHovering(false)}
          />
        )}
      </>
    );
  };

  // this will re-order the items
  const onDragEnd = (result) => {
    if (!result.destination) return;

    const sIndex = result.source.index;
    const dIndex = result.destination.index;

    // remove source item
    const withoutSource = data.filter((itm, k) => k !== sIndex);

    // add source item next to destination item
    onChange([
      ...withoutSource.slice(0, dIndex),
      data[sIndex],
      ...withoutSource.slice(dIndex),
    ]);
  };

  return (
    <DragDropContext onDragEnd={onDragEnd}>
      <Droppable droppableId="droppable">
        {(provided, snapshot) => (
          <div
            data-testid="Properties-SortableList"
            ref={provided.innerRef}
            {...provided.droppableProps}
            style={style}
          >
            {data.map((itm, key) => (
              <>
                <Draggable
                  key={key}
                  draggableId={itm.id}
                  index={key}
                  isDragDisabled={!canDragAndDrop}
                >
                  {(provided, snapshot) => (
                    <div
                      key={key}
                      ref={provided.innerRef}
                      {...provided.draggableProps}
                      {...provided.dragHandleProps}
                    >
                      <SortableItem {...itm} iKey={key} />
                    </div>
                  )}
                </Draggable>
              </>
            ))}
            {provided.placeholder}
            {canAdd && <AddButtonComponent onAdd={onAdd} name={name} />}
          </div>
        )}
      </Droppable>
    </DragDropContext>
  );
};

export default SortableList;
