import * as React from "react";
import { render, screen, fireEvent, cleanup } from "@testing-library/react";
import ColorPicker from "./ColorPicker";

const props = {
  color: "blue",
  text: "My Awesome Color Picker",
  value: "",
  onChange: () => console.log("Color Changed"),
};

describe("<ColorPicker /> Rendering >", () => {
  afterEach(cleanup);

  it(`Should render the component in the dom`, () => {
    // render the color picker
    render(<ColorPicker {...props} />);
    // test that in on the dom
    expect(screen.queryByTestId("ColorPicker")).toBeTruthy();
  });

  it(`Should show the value in the color prop in the main color picker as a highlighted color and/or a hex value in the picker`, () => {
    const { container, debug } = render(
      <ColorPicker {...props} color="pink" showHex />
    );
    expect(screen.getByTestId("ColorPicker-color").value).toBe("pink");
  });

  it(`Should show the value from the text prop in the section title`, () => {
    const { container, debug, getAllByText } = render(
      <ColorPicker {...props} />
    );
    expect(screen.getAllByText(props.text)).not.toBe(null);
  });

  it(`Should show a collection of colors as provided by the colorArray prop`, () => {
    const colorArray = ["#FFF", "#CCCCCC", "#F1C40F", "#F39C12", "#E67E22"];
    render(<ColorPicker {...props} colorArray={colorArray} />);

    colorArray.forEach((color) => {
      expect(screen.queryByTitle(color)).toBeTruthy();
    });
  });

  it(`Should show a default collection of colors if no colors are provided in a colorArray prop`, () => {
    render(<ColorPicker {...props} />);
    expect(screen.queryByTestId("ColorPicker-Circles")).toBeTruthy();
  });

  it(`Should show the array of circles when the showCircles prop is true`, () => {
    render(<ColorPicker {...props} showCircles />);
    expect(screen.queryByTestId("ColorPicker-Circles")).toBeTruthy();
  });

  it(`Should show the horizontal slider hue picker when the showSlider prop is true`, () => {
    render(<ColorPicker {...props} showSlider />);
    expect(screen.queryByTestId("ColorPicker-Slider")).toBeTruthy();
  });

  it(`Should show the hex picker when the showHex prop is true`, () => {
    render(<ColorPicker {...props} showHex />);
    expect(screen.queryByTestId("ColorPicker-Hex")).toBeTruthy();
  });

  it(`Should show the the Alpha Picker when the showAlpha prop is true`, () => {
    render(<ColorPicker {...props} showAlpha />);
    expect(screen.queryByTestId("ColorPicker-Alpha")).toBeTruthy();
  });

  test.skip(`Should show a tooltip when the group is hovered over`, () => {});
  test.skip(`Should show the expanded color options when the colorExpanded prop is true`, () => {});
});

describe("<ColorPicker /> Events >", () => {
  afterEach(cleanup);
  it(`Should trigger the onChange() event with the new color when the color is changed`, () => {
    const onChange = jest.fn();
    const selectedColor = "#cc94ab";

    render(
      <ColorPicker
        {...props}
        onChange={onChange}
        colorArray={["blue", "black", selectedColor]}
      />
    );
    fireEvent.click(screen.getByTitle(selectedColor));
    // console.log(onChange.mock.calls);
    expect(onChange.mock.calls[0][0].hex).toBe(selectedColor);
  });
});

// describe("<ColorPicker /> Actions >", () => {
//   afterEach(cleanup);
//   it(`Should update the color prop with the newly selected value after one is selected`, () => {
//     // expect(true).toBe(false);
//   });
// });

describe("<ColorPicker /> Hooks >", () => {
  afterEach(cleanup);
  test.skip(`Should trigger the _onColorPickerRender_ plugin hook when the Color Picker component is being rendered`, () => {});

  test.skip(`Should trigger the _onColorPickerChange_ plugin hook when the color is changing`, () => {});
});
